import re

from PyPDF2 import PdfReader

reader = PdfReader("中国计算机学会推荐国际学术会议和期刊目录-2019.pdf")
number_of_pages = len(reader.pages)
text = ''
for page in reader.pages[1:]:
    text += page.extract_text()
lines = text.split('\n')
merged_lines = []
for line in lines:
    if re.search(r"[\u4e00-\u9fa5]", line):  # 是中文开头？
        merged_lines.append(line)  # 加行！
    elif re.search(r"^[0-9]+ ", line):  # 是数字开头？
        merged_lines.append(line)  # 加行！
    else:  # 不是中文开头又不是数字开头？
        merged_lines[-1] += line  # 接在上一行后面
    merged_lines[-1] = merged_lines[-1].strip()

split_lines = []
for line in merged_lines:
    if re.match(r"(^[^\u4e00-\u9fa5（）()]+)[\u4e00-\u9fa5（）]", line):
        index = re.search(r"[\u4e00-\u9fa5（）]", line).start()
        split_lines.append(line[0:index])
        split_lines.append(line[index:])
    else:
        split_lines.append(line)

data = {
    'j': {},
    'c': {}
}
current = []
for i in range(len(split_lines)):
    if re.search(r"序号[ ]+[刊会][物议]简称[ ]+[刊会][物议][名全]称[ ]+出版社[ ]+网[址站]", split_lines[i]):
        pass
    elif re.search(r"中国计算机学会推荐国际学术[ ]*[期刊][刊物]", split_lines[i]):
        field = split_lines[i + 1]
        current = []
        data['j'][field] = current  # 跳转
    elif re.search(r"中国计算机学会推荐国际学术[ ]*[期刊][刊物]", split_lines[i - 1]):
        pass
    elif re.search(r"中国计算机学会推荐国际学术会议", split_lines[i]):
        field = split_lines[i + 1]
        current = []
        data['c'][field] = current  # 跳转
    elif re.search(r"中国计算机学会推荐国际学术会议", split_lines[i - 1]):
        pass
    else:
        current.append(split_lines[i])

splited_data = []
a, b, c = [], [], []
current = []
for dv in data.values():
    for fv in dv.values():
        for v in fv:
            if re.search(r"A类", v):
                print(v)
                current = a
            elif re.search(r"B类", v):
                print(v)
                current = b
            elif re.search(r"C类", v):
                print(v)
                current = c
            else:
                current.append(v)
import json

with open("CCF_A.py", 'w', encoding='utf8') as f:
    f.write("data = ")
    json.dump(a, f, indent=2)
with open("CCF_B.py", 'w', encoding='utf8') as f:
    f.write("data = ")
    json.dump(b, f, indent=2)
with open("CCF_C.py", 'w', encoding='utf8') as f:
    f.write("data = ")
    json.dump(c, f, indent=2)



data = [
  "1 TOCS  ACM Transactions on Computer Systems  ACM  http://dblp.uni -trier.de/db/journals/tocs/",
  "2 TOS  ACM Transactions on Storage  ACM  http://dblp.uni -trier.de/db/journals/tos/",
  "3 TCAD  IEEE Transactions on Computer -Aided Design ofIntegrated Circuits And System  IEEE  http://dblp.uni -trier.de/db/journals/tcad/",
  "4 TC  IEEE Transactions on Computers  IEEE  http://dblp.uni -trier.de/db/journals/tc/index.html",
  "5 TPDS  IEEE Transactions on Parallel and Distributed Systems  IEEE  http://dblp.uni -trier.de/db/journals/tpds/",
  "1 JSAC  IEEE Journal of Selected Areas in Communications  IEEE  http://dblp.uni -trier.de/db/journals/jsac/",
  "2 TMC  IEEE Transactions on Mobile Computing  IEEE  http://dblp.uni -trier.de/db/journals/tmc/",
  "3 TON  IEEE/ACM Transactions on Networking  IEEE/ACM  http://dblp.uni -trier.de/db/journals/ton/",
  "1 TDSC  IEEE Transactions on Dependable and Secure Computing  IEEE  http://dblp.uni -trier.de/db/journals/tdsc/",
  "2 TIFS  IEEE Transactions on Information Forensics and Security  IEEE  http://dblp.uni -trier.d e/db/journals/tifs/",
  "3  Journal of Cryptology  Springer  http://dblp.uni -trier.de/db/journals/joc/  ",
  "1 TOPLAS  ACM Transactions on Programming Languages & Systems  ACM  http://dblp.uni -trier.de/db/journals/toplas/",
  "2 TOSEM  ACM Transactions on Software Engineering  and  Methodology  ACM  http://dblp.uni -trier.de/db/journals/tosem/",
  "3 TSE  IEEE Transactions on Software Engineering  IEEE  http://dblp.uni -trier.de/db/journals/tse/  ",
  "1 TODS  ACM Transactions on Database Systems  ACM  http://dblp.uni -trier.de/db/journals/tods/",
  "2 TOIS  ACM Transactions on Information Systems  ACM  http://dblp.uni -trier.de/db/journals/tois/",
  "3 TKDE  IEEE Transactions on Knowledge and Data Engineering  IEEE  http://dblp.uni -trier.de/db/journals/tkde/",
  "4 VLDBJ  The VLDB Journal  Springer  http://dblp.uni -trier.de/db/journals/vldb/  ",
  "1 TIT  IEEE Transactions on Information Theory  IEEE  http://dblp.uni -trier.de/db/journals/tit/",
  "2 IANDC  Information and Computation  Elsevier  http://dblp.uni -trier.de/db/journals/iandc/",
  "3 SICOMP  SIAM Journal on Computing  SIAM  http://dblp.uni -trier.de/db/journals/siamcomp/  ",
  "1 TOG  ACM Transactions on Graphics  ACM  http://dblp.uni -trier.de/db/journals/tog/",
  "2 TIP  IEEE Transactions on Image Processing  IEEE  http://dblp.uni -trier.de/db/journals/tip/",
  "3 TVCG  IEEE Transactions on Visualization and Computer Graphics  IEEE  http://dblp.uni -trier.de/db/journals/tvcg/  ",
  "1 AI Artificial Intelligence  Elsevier  http://dblp.uni -trier.de/db/journals/ai/",
  "2 TPAMI  IEEE Trans on Pattern Analysis and MachineIntelligence  IEEE  http://dblp.uni -trier.de/db/journals/pami/",
  "3 IJCV  International Journal of Computer Vision  Springer  http://dblp.uni -trier.de/db/journals/ijcv/",
  "4 JMLR  Journal of Machine Learning Research  MIT Press  http://dblp.uni -trier.de/db/journals/jmlr/",
  "1 TOCHI  ACM Transactions on Computer -HumanInteraction  ACM  http://dblp.uni -trier.de/db/journals/tochi/",
  "2 IJHCS  International Journal of Human Computer Studies  Elsevier  http://dblp.uni -trier.de/db/journals/ijmms/  ",
  "1 JACM  Journal of the ACM  ACM  http://dblp.uni -trier.de/db/journals/jacm/",
  "2 Proc. IEEE  Proceedings of the IEEE  IEEE  http://dblp.uni -trier.de/db/journals/pieee/",
  "1 PPoPP  ACM SIGPLAN Symposium on Principles &Practice of Parallel Programming  ACM  http://dblp.uni -trier.de/db/conf/ppopp/",
  "2 FAST  Conference on File and Storage Technologies  USENIX  http://dblp.uni -trier.de/db/conf/fast/",
  "3 DAC  Design Automation Conference  ACM  https://dblp.uni -trier.de/db/conf/dac/",
  "4 HPCA  High  Performance Computer Architecture  IEEE  http://dblp.uni -trier.de/db/conf/hpca/",
  "5 MICRO  IEEE/ACM International Symposium onMicroarchitecture  IEEE/ACM  https://dblp.uni -trier.de/db/conf/micro/",
  "6 SC International Conference for High PerformanceComputing, Networking, Storage, and Analysis  IEEE  http://dblp.uni -trier.de/db/conf/sc/",
  "7 ASPLOS  International Conference on Architectural Supportfor Programming Languages and Operating Systems  ACM  http://dblp.uni -trier.de/db/conf/asplos/",
  "8 ISCA  International Symposium on Computer Architecture  ACM /IEEE  http://dblp.uni -trier.de/db/conf/isca/",
  "9 USENIX ATC  USENIX Annul Technical Conference  USENIX  http://dblp.uni -trier.de/db/conf/usenix/index.html  ",
  "1 SIGCOMM  ACM International Conference on Applications,Technologies, Architectures, and Protocols forComputer Communication  ACM  http://dblp.uni -trier.de/db/conf/sigcomm/index.html",
  "2 Mobi Com  ACM International Conference on MobileComputing and Networking  ACM  http://dblp.uni -trier.de/db/conf/mobicom/",
  "3 INFOCOM  IEEE International Conference on ComputerCommunications  IEEE  http://dblp.uni -trier.de/db/conf/infocom/",
  "4 NSDI  Symposium on Network System Design andImplementation  USENIX  http://dblp.uni -trier.de/db/conf/nsdi/",
  "1 CCS  ACM Conference on Computer andCommunications Security  ACM  http://dblp.uni -trier.de/db/conf/ccs/",
  "2 EUROCRYPT  European Cryptology Conference  Springer  http://dblp.uni -trier.de/db/conf/eurocrypt/",
  "3 S&P  IEEE Symposium on Security and Privacy  IEEE  http://dblp.uni -trier.de/db/conf/sp/",
  "4 CRYPTO  International Cryptology Conference  Springer  http://dblp.uni -trier.de/db/conf/crypto/",
  "5 USENIX Security  Usenix Security Symposium  USENIX Association  http://dblp.uni -trier.de/db/conf/uss/  ",
  "1 PLDI  ACM SIGPLAN Conference on Programming Language Design &Implementation  ACM  http://dblp.uni -trier.de/db/conf/pldi/",
  "2 POPL  ACM SIGPLAN -SIGACT Symposium on Principles of ProgrammingLanguages  ACM  http://dblp.uni -trier.de/db/conf/popl/",
  "3 FSE/ESEC  ACM SIGSOFT Symposium on the Foundation of Software Engineering/European Software Engineering Conference  ACM  http://dblp.uni -trier.de/db/conf/sigsoft/",
  "4 SOSP  ACM Symposium on Operating Systems Principles  ACM  http://dblp.uni -trier.de/db/conf/sosp/",
  "5 OOPSLA  Conference on Object -Oriented Programming Systems, Languages, andApplications  ACM  http://dblp.uni -trier.de/db/conf/oopsla/",
  "6 ASE  International Conference on Automated Software Engineering  IEEE/ACM  http://dblp.uni -trier.de/db/conf/kbse/",
  "7 ICSE  International Conference on Software Engineering  ACM/IEEE  http://dblp.uni -trier.de/db/conf/icse/",
  "8 ISSTA  International Symposium on Software Testing and Analysis  ACM  http://dblp.uni -trier.de/db/conf/issta/",
  "9 OSDI  USENIX Symposium on Operating Systems Design and Implementations  USENIX  http://dblp.uni -trier.de/db/conf/osdi/",
  "1 SIGMOD  ACM Conference on Management of Data  ACM  http://dblp.uni -trier.de/db/conf/sigmod/",
  "2 SIGKDD  ACM Knowledge Discovery and Data Mining  ACM  http://dblp.uni -trier.de/db/conf/kdd/",
  "3 ICDE  IEEE International Conference on DataEngineering  IEEE  http://dblp.uni -trier.de/db/conf/icde/",
  "4 SIGIR  International Conference on Research onDevelopment in Information Retrieval  ACM  http://dblp.uni -trier.de/db/conf/sigir/",
  "5 VLDB  International Conference on Very Large Data Bases  MorganKaufmann/ACM  http://dblp.uni -trier.de/db/conf/vldb/",
  "1 STOC  ACM Symposium on the Theory of Computing  ACM  http://dblp.uni -trier.de/db/conf/stoc/",
  "2 SODA  ACM -SIAM Symposium on Discrete Algorithms  SIAM  http://dblp.uni -trier.de/db/conf/soda/",
  "3 CA V  Computer Aided Verification  Springer  http://dblp.uni -trier.de/db/conf/cav/",
  "4 FOCS  IEEE Annual Symposium on Foundations of Computer Science  IEEE  http://dblp.uni -trier.de/db/conf/focs/",
  "5 LICS  IEEE Symposium on Logic in Computer Science  IEEE  http://dblp.uni -trier.de/db/conf/lics/",
  "1 ACM MM  ACM International Conference on Multimedia  ACM  http://dblp.uni -trier.de/db/conf/mm/",
  "2 SIGGRAPH  ACM SIGGRAPH Annual Conference  ACM  http://dblp.uni -trier.de/db/conf/siggraph/index.html",
  "3 VR  IEEE Virtual Reality  IEEE  http://dblp.uni -trier.de/db/conf/vr/",
  "4 IEEE VIS  IEEE Visualization Conference  IEEE  http://dblp.uni -trier.de/db/conf/visualization/index.html  ",
  "1 AAAI  AAAI Conference on ArtificialIntelligence  AAAI  http://dblp.uni -trier.de/db/conf/aaai/",
  "2 NeurIPS  Annual Conference on NeuralInformation Processing Systems  MIT Press  http://dblp.uni -trier.de/db/conf/nips/",
  "3 ACL  Annual Meeting of theAssociation for ComputationalLinguistics  ACL  http://dblp.uni -trier.de/db/conf/acl/",
  "4 CVPR  IEEE Conference on ComputerVision and Pattern Recognition  IEEE  http://dblp.uni -trier.de/db/conf/cvpr/",
  "5 ICCV  International Conference onComputer Vision  IEEE  http://dblp.uni -trier.de/db/conf/iccv/",
  "6 ICML  International Conference onMachine Learning  ACM  http://dblp.uni -trier.de/db/conf/icml/",
  "7 IJCAI  International Joint Conference on Artificial Intelligence  Morgan Kaufmann  http://dblp.uni -trier.de/db/conf/ijcai/  ",
  "1 CSCW  ACM Conference on Computer SupportedCooperative Work and Social Computing  ACM  http://dblp.uni -trier.de/db/conf/cscw",
  "2 CHI  ACM Conference on Human Factors inComputing Systems  ACM  http://dblp.uni -trier.de/db/conf/chi",
  "3 UbiComp  ACM International Conference on UbiquitousComputing  ACM  http://dblp.uni -trier.de/db/conf/huc/",
  "1 WWW  International World Wide Web Conferences  ACM  http://dblp.uni -trier.de/db/conf/www/",
  "2 RTSS  Real -Time Systems Symposium  IEEE  http://dblp.uni -trier.de/db/conf/rtss/  "
]


import re
from typing import Optional
import os
from datetime import datetime, timedelta
import xml.etree.ElementTree as ElementTree

import aiohttp
import logging
from aiofile import async_open
from asyncio import Semaphore

logger = logging.getLogger("downloader")

http_sem = Semaphore(8)
file_sem = Semaphore(512)


def getenv_int(key) -> int:
    cache_days = os.getenv(key)
    if cache_days is not None:
        try:
            return int(cache_days)
        except:
            pass
    return None


def get_cache_datetime(path) -> datetime:
    return datetime.fromtimestamp(os.path.getmtime(path))


async def download_person(pid: str) -> Optional[ElementTree.Element]:
    cache_days = getenv_int('DBLP_CRAWLER_MAX_CACHE_DAYS_PERSON')
    cache_days = cache_days if cache_days is not None else 30
    return await download_item("pid/" + pid + ".xml", cache_days)


async def download_journal_list(pid: str) -> Optional[ElementTree.Element]:
    cache_days = getenv_int('DBLP_CRAWLER_MAX_CACHE_DAYS_JOURNAL_LIST')
    cache_days = cache_days if cache_days is not None else 300
    return await download_item(pid + "/index.xml", cache_days)


async def download_journal(pid: str) -> Optional[ElementTree.Element]:
    cache_days = getenv_int('DBLP_CRAWLER_MAX_CACHE_DAYS_JOURNAL')
    cache_days = cache_days if cache_days is not None else 300
    return await download_item(re.sub(r"\.html$", ".xml", pid), cache_days)


async def download_item(path: str, cache_days: int) -> Optional[ElementTree.Element]:
    save_path = os.path.join("save", path)
    if os.path.isfile(save_path):
        if datetime.now() < get_cache_datetime(save_path) + timedelta(days=cache_days):
            async with file_sem:
                try:
                    async with async_open(save_path, 'r') as f:
                        logger.info("use cache: %s" % save_path)
                        html = await f.read()
                        data = ElementTree.fromstring(html)
                        return data
                except:
                    logger.debug(" no cache: %s" % save_path)
        else:
            logger.info('cache outdated: %s' % save_path)

    url = "https://dblp.org/" + path
    async with http_sem:
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url) as response:
                    logger.info(" download: %s" % path)
                    html = await response.text()
                    data = ElementTree.fromstring(html)
                    os.makedirs(os.path.dirname(save_path), exist_ok=True)
                    async with async_open(save_path, 'w') as f:
                        await f.write(html)
                    return data
        except Exception as e:
            logger.error("invalid response: %s" % e)
    return None



import abc
import asyncio
import json
import logging

import networkx as nx

from dblp_crawler import Graph

logger = logging.getLogger("graph")


class NetworkxGraph(Graph, metaclass=abc.ABCMeta):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.graph: nx.MultiGraph = nx.MultiGraph()

    def summarize_person(self, a, person):  # 构建summary
        self.graph.add_node(a, person=self.persons[a])  # 把作者信息加进图里

    def summarize_publication(self, a, b, publication):  # 构建summary
        self.graph.add_edge(a, b, key=publication.key(), publication=publication)  # 把边加进图里

    def multi_graph_summary(self):
        """输出一个 networkx.MultiGraph，节点对应作者，每条边对应一篇论文，作者间可有多条边"""
        self.graph = nx.MultiGraph()
        self.summarize()
        return self.graph

    def summary_person(self, person, publications):  # 构建summary
        return dict(
            person=person, publications=list(publications.values())
        )

    def summary_cooperation(self, a, b, publications):  # 构建summary
        a_pub = set(p.key() for p in a.publications())
        b_pub = set(p.key() for p in b.publications())
        return dict(publications=list(publications.values()), cooperation=list(a_pub.intersection(b_pub)))

    def graph_summary(self):
        """输出一个 networkx.Graph，节点对应作者，每条边对应多篇论文，作者间仅有一条边"""
        g = self.multi_graph_summary()
        gg = nx.Graph()
        authors = {}
        for (a, b, d) in g.edges(data=True):  # 遍历所有文章
            publication = d['publication']
            # 把文章加进边信息里
            data = gg.get_edge_data(a, b)
            if data is None or "publications" not in data:
                data = {"publications": {}}
            data["publications"][publication.key()] = publication
            gg.add_edge(a, b, **data)
            # 把文章加进作者信息里
            if a not in authors:
                authors[a] = {}
            authors[a][publication.key()] = publication
            if b not in authors:
                authors[b] = {}
            authors[b][publication.key()] = publication
        for pid, publications in authors.items():
            gg.add_node(pid, **self.summary_person(g.nodes[pid]['person'], publications))
        for (a, b, d) in list(gg.edges(data=True)):  # 遍历所有文章
            gg.add_edge(a, b, **self.summary_cooperation(
                g.nodes[a]['person'], g.nodes[b]['person'], d["publications"]
            ))
        return gg

    def dict_summary(self):

        g = self.graph_summary()

        def update(pub, selected=False):
            if pub.key() not in publications:
                publications[pub.key()] = pub.__dict__()
            publications[pub.key()] = {**publications[pub.key()], **pub.__dict__()}
            if selected:
                publications[pub.key()]['selected'] = True

        nodes, edges, publications = {}, {}, {}
        for k, d in g.nodes(data=True):
            nodes[k] = {
                'id': k, 'label': d['person'].name(),
                'person': d['person'].__dict__(),
                'publications': [pub.key() for pub in d['publications']]
            }
            for pub in d['publications']:
                update(pub, True)
            for pub in d['person'].publications():
                update(pub)
        for u, v, d in g.edges(data=True):
            edges[json.dumps({'from': u, 'to': v})] = {
                'from': u, 'to': v,
                'publications': [pub.key() for pub in d['publications']],
                'cooperation': d['cooperation']
            }
            for pub in d['publications']:
                update(pub, True)
        return dict(nodes=nodes, edges=edges, publications=publications)



import abc
import asyncio
import logging
from typing import Optional

from neo4j import Session

from dblp_crawler import Graph, DBLPPerson, Publication

logger = logging.getLogger("graph")


def add_person(tx, person: DBLPPerson, added_pubs: set):
    tx.run("MERGE (a:Person {pid: $pid, name: $name})", pid=person.pid(), name=person.name())
    for publication in person.publications():
        if publication.key() in added_pubs:
            continue
        added_pubs.add(publication.key())
        tx.run("MATCH (a:Person) WHERE a.pid = $pid "
               "MERGE (p:Publication {key:$key, title:$title, journal_key:$journal_key, journal:$journal, year:$year}) "
               "MERGE (a)-[:WRITE]->(p)",
               pid=person.pid(),
               key=publication.key(),
               title=publication.title(),
               journal_key=publication.journal_key() or "",
               journal=publication.journal() or "",
               year=publication.year())


def add_edge(tx, a: str, b: str, publication: Publication):
    tx.run("MATCH (a:Person) WHERE a.pid = $a "
           "MATCH (b:Person) WHERE b.pid = $b "
           "MERGE (p:Publication {key:$key, title:$title, journal_key:$journal_key, journal:$journal, year:$year, selected: true}) "
           "MERGE (a)-[:WRITE]->(p)"
           "MERGE (b)-[:WRITE]->(p)",
           a=a, b=b,
           key=publication.key(),
           title=publication.title(),
           journal_key=publication.journal_key(),
           journal=publication.journal(),
           year=publication.year())


class Neo4jGraph(Graph, metaclass=abc.ABCMeta):
    def __init__(self, session: Session, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.session = session
        self.added_pubs = set()

    def summarize_person(self, a: str, person: Optional[DBLPPerson]):  # 构建summary
        if person is not None:
            self.session.execute_write(add_person, person, self.added_pubs)  # 把作者信息加进图里

    def summarize_publication(self, a: str, b: str, publication: Publication):  # 构建summary
        self.session.execute_write(add_edge, a, b, publication)  # 把边加进图里



import argparse
import asyncio
from typing import Iterable
import logging

from dblp_crawler import Publication
from dblp_crawler.keyword import Keywords
from dblp_crawler.keyword.arg import add_argument as add_argument_kw, parse_args as parse_args_kw
from dblp_crawler.arg import add_argument_pid, add_argument_journal, parse_args_pid_journal
from dblp_crawler.summarizer.networkx import NetworkxGraph
from dblp_crawler.summarizer.neo4j import Neo4jGraph

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('dblp_crawler')

parser = argparse.ArgumentParser()

parser.add_argument("-y", "--year", type=int, help="Only crawl the paper after the specified year.", default=2000)
add_argument_kw(parser)
add_argument_pid(parser)
add_argument_journal(parser)


def func_parser(parser):
    args = parser.parse_args()
    year = args.year
    keywords = parse_args_kw(parser)
    pid_list, journal_list = parse_args_pid_journal(parser)
    logger.info(f"Specified keyword rules: {keywords.rules}")
    logger.info(f"Specified pid_list for init: {pid_list}")
    logger.info(f"Specified journal_list for init: {journal_list}")
    return year, keywords, pid_list, journal_list


def filter_publications_at_crawler(publications, keywords: Keywords):
    for publication in publications:
        if keywords.match_words(publication.title()):
            yield publication


def filter_publications_at_output(publications, year: int, keywords: Keywords):
    for publication in publications:
        if publication.year() >= year and keywords.match(publication.title()):
            yield publication


async def bfs_to_end(graph, limit: int = 0):
    while min(*(await graph.bfs_once())) > 0 and (limit != 0):
        logger.info("Still running......")
        limit -= 1


subparsers = parser.add_subparsers(help='sub-command help')


# --------- for NetworkxGraph ---------

class NetworkxGraphDefault(NetworkxGraph):
    def __init__(self, year, keywords, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.year = year
        self.keywords = keywords

    def filter_publications_at_crawler(self, publications: Iterable[Publication]) -> Iterable[Publication]:
        yield from filter_publications_at_crawler(publications, self.keywords)

    def filter_publications_at_output(self, publications: Iterable[Publication]) -> Iterable[Publication]:
        yield from filter_publications_at_output(publications, self.year, self.keywords)


parser_nx = subparsers.add_parser('networkx', help='Write results to a json file.')
parser_nx.add_argument("--dest", type=str, required=True, help=f'Path to write results.')


def func_parser_nx(parser):
    year, keywords, pid_list, journal_list = func_parser(parser)
    args = parser.parse_args()
    dest = args.dest
    logger.info(f"Specified dest: {dest}")
    g = NetworkxGraphDefault(year=year, keywords=keywords, pid_list=pid_list, journal_list=journal_list)
    asyncio.get_event_loop().run_until_complete(bfs_to_end(g))
    summary = g.dict_summary()

    import json

    with open(dest, 'w', encoding='utf8') as f:
        json.dump(summary, fp=f, indent=2)


parser_nx.set_defaults(func=func_parser_nx)


# --------- for Neo4jGraph ---------


class Neo4jGraphDefault(Neo4jGraph):
    def __init__(self, year, keywords, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.year = year
        self.keywords = keywords

    def filter_publications_at_crawler(self, publications: Iterable[Publication]) -> Iterable[Publication]:
        yield from filter_publications_at_crawler(publications, self.keywords)

    def filter_publications_at_output(self, publications: Iterable[Publication]) -> Iterable[Publication]:
        yield from filter_publications_at_output(publications, self.year, self.keywords)


parser_n4j = subparsers.add_parser('neo4j', help='Write result to neo4j database')
parser_n4j.add_argument("--auth", type=str, default=None, help=f'Auth to neo4j database.')
parser_n4j.add_argument("--uri", type=str, required=True, help=f'URI to neo4j database.')


def func_parser_n4j(parser):
    from neo4j import GraphDatabase
    year, keywords, pid_list, journal_list = func_parser(parser)
    args = parser.parse_args()
    logger.info(f"Specified uri and auth: {args.uri} {args.auth}")

    with GraphDatabase.driver(args.uri, auth=args.auth) as driver:
        with driver.session() as session:
            g = Neo4jGraphDefault(
                year=year, keywords=keywords, session=session,
                pid_list=pid_list, journal_list=journal_list)
            asyncio.get_event_loop().run_until_complete(bfs_to_end(g))
            logger.info(f"Summarizing to: {args.uri}")
            g.summarize()


parser_n4j.set_defaults(func=func_parser_n4j)

# --------- Run ---------
args = parser.parse_args()
args.func(parser)



import argparse
import importlib
from . import Keywords


def add_argument(parser: argparse.ArgumentParser, *flags, dest: str = 'keyword', required=True) -> None:
    if len(flags) <= 0:
        flags = ['-k', '--keyword']
    parser.add_argument(
        *flags, dest=dest, action='append', required=required,
        help=f'Specify keyword rules.'
    )


def parse_args(parser: argparse.ArgumentParser, dest: str = 'keyword'):
    args = parser.parse_args()
    keywords = Keywords()
    for s in args.__getattribute__(dest):
        try:
            s = eval(s)
            if isinstance(s, str):
                keywords.add_rule(s)
            else:
                keywords.add_rule(*s)
        except:
            keywords.add_rule(s)
    return keywords



from .utils import map_publications, map_person_publications, map_node, map_edge, map_cooperation


def drop_old_publications(summary, year):
    def callback(_, publication):
        if "selected" in publication and publication["selected"]:
            return publication
        if publication["year"] >= year:
            return publication

    return map_publications(summary, callback)


def drop_old_person_publications(summary, year):
    def callback(_, publication):
        if publication["year"] >= year:
            return publication

    return map_person_publications(summary, callback)


def drop_old_cooperation(summary, year):
    def callback(_, publication):
        if publication["year"] >= year:
            return publication

    return map_cooperation(summary, callback)


def drop_nodes_by_all_publications(summary, n):
    def callback(_, node):
        if len(node["person"]["publications"]) >= n:
            return node

    return map_node(summary, callback)


def drop_edges_by_all_publications(summary, n):
    def callback(_, edge):
        if len(edge["cooperation"]) >= n:
            return edge

    return map_edge(summary, callback)



import argparse
import logging
import json
import importlib
from dblp_crawler.filter import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('dblp_crawler.filter')

parser = argparse.ArgumentParser()

parser.add_argument("-i", "--input", type=str, required=True, help=f'Input file path.')
parser.add_argument("-o", "--output", type=str, required=True, help=f'Output file path.')
parser.add_argument("-f", "--filter", action='append', required=True, help=f'Filter functions.')
args = parser.parse_args()
with open(args.input) as source:
    summary = json.load(source)
    for fs in args.filter:
        logger.info("Applying: %s" % fs)
        summary = eval(fs)(summary)
    with open(args.output, 'w', encoding='utf8') as destin:
        json.dump(summary, destin, indent=2)



import re
from typing import Optional
from .CCF_A import data as CCF_A
from .CCF_B import data as CCF_B
from .CCF_C import data as CCF_C

CCF = ''


def key(r: str, ccf: str) -> Optional[str]:
    urls = re.findall(r"https*://.*$", r)
    if len(urls) != 1:
        return None
    url = urls[0].replace(" ", "")
    search = re.search(r"https*://dblp\.uni-trier.de/(.+?)$", url)
    if search is None:
        return None
    return "/".join(search.group(1).split("/")[0:3])


CCF_A: list[str] = list(filter(None, (key(r, 'A') for r in CCF_A)))
CCF_B: list[str] = list(filter(None, (key(r, 'B') for r in CCF_B)))
CCF_B += [
    "db/journals/pe",
    "db/journals/tissec",
    "db/journals/cogsci",
    "db/conf/hotchips"
]
CCF_C: list[str] = list(filter(None, (key(r, 'C') for r in CCF_C)))



import re


class Keywords:
    def __init__(self) -> None:
        self.rules: set[tuple[str, ...]] = set()
        self.words: set[str] = set()

    def add_rule(self, *rule: str) -> None:
        """与关系的单词列"""
        rule = tuple(frozenset(word.lower() for word in rule))
        self.rules.add(rule)
        self.words = self.words.union(rule)

    def add_rule_list(self, *rule_list: set[str]) -> None:
        """多个与关系的单词列"""
        for rule in rule_list:
            self.add_rule(*rule)

    def add_word_rules(self, *words: str) -> None:
        self.add_rule_list(*list(set(word, ) for word in words))

    def match(self, sentence: str) -> bool:
        sentence = sentence.lower()
        words = set(re.findall(r"\w+", sentence))
        for rule in self.rules:  # 只要有一个rule能匹配上就返回True
            if set(rule).issubset(words):  # 同时包含rule中所列的所有关键词就返回True
                return True
        return False

    def match_words(self, sentence: str) -> bool:
        sentence = sentence.lower()
        # 所有rule全切成一个个单词，只要包含其中一个单词就返回True
        return len(set(re.findall(r"\w+", sentence)).intersection(self.words)) > 0


if __name__ == "__main__":
    kw: Keywords = Keywords()
    kw.add_rule_list({"super", "resolution"}, {"content", "aware"})
    kw.add_rule("video")
    kw.add_rule("edge", "computing")
    print(kw.match("An adaptive clustering-based evolutionary algorithm for many-objective optimization problems"))
    print(kw.match(
        "Multi-resolution representation with recurrent neural networks application for streaming time series in IoT"))
    print(kw.match(
        "High-Definition Video Compression System Based on Perception Guidance of Salient Information of a Convolutional Neural Network and HEVC Compression Domain"))
    print(kw.match("Resource Provision and Allocation Based on Microeconomic Theory in Mobile Edge Computing"))



data = [
  "1 TACO  ACM Transactions on Architecture and Code Optimization  ACM  http://dblp.uni -trier.de/db/journals/taco/",
  "2 TAAS  ACM Transactions on Autonomous and Adaptive Systems  ACM  http://dblp.uni -trier.de/db/journals/taas/",
  "3 TODAES  ACM Transactions on Design Automation of Electronic Systems  ACM  http://dblp.uni -trier.de/db/journals/todaes/",
  "4 TECS  ACM Transactions on Embedded Computing Systems  ACM  http://dblp.uni -trier.de/db/journals/tecs/",
  "5 TRETS  ACM Transactions on Reconfigurable Technology and Systems  ACM  http://dblp.uni -trier.de/db/journals/trets/",
  "6 TVLSI  IEEE Transactions on Very Large Scale Integration (VLSI) Systems  IEEE  http://dblp.uni -trier.de/db/journals/tvlsi/",
  "7 JPDC  Journal of Parallel and Distributed Computing  Elsevier  http://dblp.uni -trier.de/db/journals/jpdc/",
  "8 JSA  Journal of Systems Architecture: Embedded Software Design  Elsevier  http://dblp.uni -trier.de/db/journals/jsa/",
  "9 PARCO  Parallel Computing  Elsevier  http://dblp.uni -trier.de/db/conf/parco/",
  "10  Performance Evaluation: An International Journal  Elsevier  http://www.journals.elsevier.com/performance -evaluation/  ",
  "1 TOIT  ACM Transactions on Internet Technology  ACM  http://dblp.uni -trier.de/db/journals/toit/",
  "2 TOMCCAP  ACM Transactions on Multimedia Computing,Communications and Applications  ACM  http://dblp.uni -trier.de/db/journals/tomccap/",
  "3 TOSN  ACM Transactions on Sensor Networks  ACM  http://dblp.uni -trier.de/db/journals/tosn/",
  "4 CN  Computer Networks  Elsevier  http://dblp.uni -trier.de/db/journals/cn/",
  "5 TC OM  IEEE Transactions on Communications  IEEE  http://dblp.uni -trier.de/db/journals/tcom/",
  "6 TWC  IEEE Transactions on Wireless Communications  IEEE  http://dblp.uni -trier.de/db/journals/twc/",
  "1 TOPS  ACM Transactions on Privacy and Security  ACM  https://tops.acm.org/",
  "2  Computers & Security  Elsevier  http://dblp.uni -trier.de/db/journals/compsec/",
  "3  Designs, Codes and Cryptography  Springer  http://dblp.uni -trier.de/db/journals/dcc/",
  "4 JCS  Journal of Computer Security  IOS Press  http://dblp.uni -trier.de/db/journals/jcs/",
  "1 ASE  Automated Software Engineering  Springer  http://dblp.uni -trier.de/db/journals/ase/",
  "2 ESE  Empirical Software Engineering  Springer  http://dblp.uni -trier.de/db/journals/ese/",
  "3 TSC  IEEE Transactions on Service Computing  IEEE  http://dblp.uni -trier.de/db/journals/tsc/",
  "4 IETS  IET Software  IET  https://dblp.uni -trier.de/db/journals/iet -sen/",
  "5 IST  Information and Software Technology  Elsevier   http://dblp.uni -trier.de/db/journals/infsof/index.html",
  "6 JFP  Journal of Functional Programming  CambridgeUniversity Press  http://dblp.uni -trier.de/db/journals/jfp/",
  "7  Journal of Software: Evolution and Process  Wiley  http://dblp.uni -trier.de/db/journals/smr/",
  "8 JSS  Journal of Systems and Software  Elsevier   http://dblp.uni -trier.de/db/journals/jss/",
  "9 RE  Requirements Engineering  Springer  http://dblp.uni -trier.de/db/journals/re/",
  "10 SCP  Science of Computer Programming  Elsevier  http://dblp.uni -trier.de/db/journals/scp/",
  "11 SoSyM  Software and System Modeling  Springer  http://dblp.uni -trier.de/db/journals/sosym/",
  "12 STVR  Software Testing, Verification and Reliability  Wiley  http://dblp.uni -trier.de/db/journals/stvr/index.html",
  "13 SPE  Software: Practice and Experience  Wiley  http://dblp.uni -trier.de/db/journals/spe/  ",
  "1 TKDD  ACM Transactions on KnowledgeDiscovery from Data  ACM  http://dblp.uni -trier.de/db/journals/tkdd/",
  "2 TWEB  ACM Transactions on the Web  ACM  http://dblp.uni -trier.de/db/journals/tweb/",
  "3 AEI  Advanced Engineering Informatics  Elsevier  http://dblp.uni -trier.de/db/journals/aei/",
  "4 DKE  Data and Knowledge Engineering  Elsevier  http://dblp.uni -trier.de/db/journals/dke/",
  "5 DMKD  Data Mining and Knowledge Discovery  Springer  http://dblp.uni -trier.de/db/journals/datamine/",
  "6 EJIS  European Journal of InformationSystems  Springer  http://dblp.uni -trier.de/db/journals/ejis/",
  "7  GeoInformatica  Springer  http://dblp.uni -trier.de/db/journals/geoinformatica/",
  "8 IPM  Information Processing and Management  Elsevier  http://dblp.uni -trier.de/db/journals/ipm/",
  "9  Information Sciences  Elsevier  http://dblp.uni -trier.de/db/journals/isci/",
  "10 IS Information Systems  Elsevier  http://dblp.uni -trier.de/db/journals/is/",
  "11 JASIST  Journal of the American Society forInformation Science and Technology  American Society forInformation Science andTechnology  http://dblp.uni -trier.de/db/journals/jasis/",
  "12 JWS  Journal of Web Semantics  Elsevier  http://dblp.uni -trier.de/db/journals/ws/",
  "13 KAIS  Knowledge and Information Systems  Springer  http://dblp.uni -trier.de/db/journals/kais/  ",
  "1 TALG  ACM Transactions on Algorithms  ACM  http://dblp.uni -trier.de/db/journals/talg/",
  "2 TOCL  ACM Transactions on Computational Logic  ACM  http://dblp.uni -trier.de/db/journals/tocl/",
  "3 TOMS  ACM Transactions on Mathematical Software  ACM  http://dblp.uni -trier.de/db/journals/toms/",
  "4 Algorithmica  Algorithmica  Springer  http://dblp.uni -trier.de/db/journals/algorithmica/",
  "5 CC  Computational Complexity  Springer  http://dblp.uni -trier.de/db/journals/cc/",
  "6 FAC  Formal Aspects of Computing  Springer  http://dblp.uni -trier.de/db/journals/fac/",
  "7 FMSD  Formal Methods in System Design  Springer  http://dblp.uni -trier.de/db/journals/fmsd/",
  "8 INFORMS  INFORMS Journal on Computing  INFORMS  http://dblp.uni -trier.de/db/journals/informs/",
  "9 JCSS  Journal of Computer and System Sciences  Elsevier  http://dblp.uni -trier.de/db/journals/jcss/",
  "10 JGO  Journal of Global Optimization  Springer  http://dblp.uni -trier.de/db/journals/jgo/",
  "11 JSC  Journal of Symbolic Computation  Elsevier  http://dblp.uni -trier.de/db/journals/jsc/",
  "12 MSCS  Mathematical Structures in Computer Science  CambridgeUniversity Press  http://dblp.uni -trier.de/db/journals/mscs/",
  "13 TCS  Theoretical Computer Science  Elsevier  http://dblp.uni -trier.de/db/journals/tcs/",
  "1 TOMCCAP  ACM Transactions on Multimedia Computing,Communications and Application  ACM  http://dblp.uni -trier.de/db/journals/tomccap/",
  "2 CAGD  Computer Aided Geometric Design  Elsevier  http://dblp.uni -trier.de/db/journals/cagd/",
  "3 CGF  Computer Graphics Forum  Wiley  http://dblp.uni -trier.de/db/journals/cgf/",
  "4 CAD  Computer -Aided Design  Elsevier  http://dblp.uni -trier.de/db/journals/cad/",
  "5 GM  Graphical Models  Elsevier  http://dblp.uni -trier.de/db/journals/cvgip/",
  "6 TCSVT  IEEE Transactions on Circuits and Systems forVideo Technology  IEEE  http://dblp.uni -trier.de/db/journals/tcsv/",
  "7 TMM  IEEE Transactions on Multimedia  IEEE  http://dblp.uni -trier.de/db/journals/tmm/",
  "8 JASA  Journal of The Acoustical Society of America  AIP  http://scitation.aip.org/content/asa/journal/jasa",
  "9 SIIMS  SIAM Journal on Imaging Sciences  SIAM  http://dblp.uni -trier.de/db/journals/siamis/",
  "10 Speech Com  Speech Communication  Elsevier  http://dblp.uni -trier.de/db/journals/speech/  ",
  "1 TAP  ACM Transactions on Applied Perception  ACM  http://dblp.uni -trier.de/db/journals/tap/",
  "2 TSLP  ACM Transactions on Speech and LanguageProcessing  ACM  http://dblp.uni -trier.de/db/journals/tslp/",
  "3 AAMAS  Autonomous Agents and Multi -Agent Systems  Springer  http://dblp.uni -trier.de/db/journals/aamas/",
  "4   Computational Linguistics  MIT Press  http://dblp.uni -trier.de/db/journals/coling/",
  "5 CVIU  Computer Vision and Image Understanding  Elsevier  http://dblp.uni -trier.de/db/journals/cviu/",
  "6 DKE  Data and Knowledge Engineering  Elsevier  http://dblp.uni -trier.de/db/journals/dke/index.html",
  "7   Evolutionary Computation  MIT Press  http://dblp.uni -trier.de/db/journals/ec/",
  "8 TAC  IEEE Transactions on Affective Computing  IEEE  http://dblp.uni -trier.de/db/journals/taffco/",
  "9 TASLP  IEEE Transactions on Audio, Speech, andLanguage Processing  IEEE  http://dblp.uni -trier.de/db/journals/taslp/",
  "10   IEEE Transactions on Cybernetics  IEEE  http://dblp.uni -trier.de/db/journals/tcyb/",
  "11 TEC  IEEE Transactions on EvolutionaryComputation  IEEE  http://dblp.uni -trier.de/db/journals/tec/",
  "12 TFS  IEEE Transactions on Fuzzy Systems  IEEE  http://dblp.uni -trier.de/db/journals/tfs/",
  "13 TNNLS  IEEE Transactions on Neural Networks andlearning systems  IEEE  http://dblp.uni -trier.de/db/journals/tnn/",
  "14 IJAR  International Journal of ApproximateReasoning  Elsevier  http://dblp.uni -trier.de/db/journals/ijar/  ",
  "15 JAIR  Journal of Artificial Intelligence Research  AAAI  http://dblp.uni -trier.de/db/journals/jair/index.html",
  "16   Journal of Automated Reasoning  Springer  http://dblp.uni -trier.de/db/journals/jar/",
  "17 JSLHR  Journal of Speech, Language, and HearingResearch  American Speech -Language HearingAssociation  http://jslhr.pubs.asha.org/",
  "18   Machine Learning  Springer  http://dblp.uni -trier.de/db/journals/ml/",
  "19   Neural Computation  MIT Press  http://dblp.uni -trier.de/db/journals/neco/",
  "20   Neural Networks  Elsevier  http://dblp.uni -trier.de/db/journals/nn/",
  "21  PR Pattern Recognition  Elsevier  http://dblp.uni -trier.de/db/conf/par/  ",
  "1 CSCW  Computer Supported Cooperative Work  Springer  http://dblp.uni -trier.de/db/journals/cscw/",
  "2 HCI  Human Computer Interaction  Taylor & Francis  http://dblp.uni -trier.de/db/journals/hhci/",
  "3  IEEE Transactions on Human -Machine Systems  IEEE  http://dblp.uni -trier.de/db/journals/thms/",
  "4 IWC  Interacting with Computers  Oxford University Press  http://dblp.uni -trier.de/db/journals/iwc/",
  "5 IJHCI  International Journal of Human -ComputerInteraction  Taylor & Francis  http://dblp.uni -trier.de/db/journals/ijhci/",
  "6 UMUAI  User Modeling and User -Adapted Interaction  Springer  http://dblp.uni -trier.de/db/journals/umuai/",
  "1  Bioinformatics  Oxford University Press  http://dblp.uni -trier.de/db/journals/bioinformatics/",
  "2  Briefings in Bioinformatics  Oxford University Press  http://dblp.uni -trier.de/db/journals/bib/",
  "3 Cognition  Cognition \uff1aInternational Journal of CognitiveScience  Elsevier  http://www.journals.elsevier.com/cognition/",
  "4 TASAE  IEEE Transactions on Automation Scienceand Engineering  IEEE  http://dblp.uni -trier.de/db/journals/tase/",
  "5 TGARS  IEEE Transactions on Geoscience andRemote Sensing  IEEE  http://dblp.uni -trier.de/db/journals/tgrs/",
  "6 TITS  IEEE Transactions on IntelligentTransportation Systems  IEEE  http://dblp.uni -trier.de/db/journals/tits/",
  "7 TMI  IEEE Transactions on Medical Imaging  IEEE  http://dblp.uni -trier.de/db/journals/tmi/",
  "8 TR  IEEE Transactions on Robotics  IEEE  http://dblp.uni -trier.de/db/journals/trob/",
  "9 TCBB  IEEE -ACM Transactions on ComputationalBiology and Bioinformatics  IEEE/ACM  http://dblp.uni -trier.de/db/journals/tcbb/",
  "10 JCST  Journal of Computer Science and Technology  SCIENCE PRESS/Springer  http://dblp.uni -trier.de/db/journals/jcst/",
  "11 JAMIA  Journal of the American Medical InformaticsAssociation  BMJ Journals  http://dblp.uni -trier.de/db/journals/jamia/",
  "12  PLOS Computational Biology  Public Library of Science  http://dblp.uni -trier.de/db/journals/ploscb/",
  "13  Science China Information Sciences  Science in ChinaPress/Springer  http://dblp.uni -trier.de/db/journals/chinaf/",
  "14  The Computer Journal  Oxford University Press  http://dblp.uni -trier.de/db/journals/cj/  ",
  "15  World Wide Web Journal  Springer  http://dblp.uni -trier.de/db/journals/wwwj/  ",
  "1 SoCC  ACM Symposium on Cloud Computing  ACM  http://dblp.uni -trier.de/db/conf/cloud/",
  "2 SPAA  ACM Symposium on Parallelism in Algorithms and Architectures  ACM  http://dblp.uni -trier.de/db/conf/spaa/",
  "3 PODC  ACM Symposium on Principles of Distributed Computing  ACM  http://dblp.uni -trier.de/db/conf/podc/",
  "4 FPGA  ACM/SIGDA International Symposium on Field -ProgrammableGate Arrays  ACM  http://dblp.uni -trier.de/db/conf/fpga/",
  "5 CGO  Code Generation and Optimization  IEEE/ACM  http://dblp.uni -trier.de/db/conf/cgo/",
  "6 DATE  Design, Automation & Test in Europe  IEEE/ACM  http://dblp.uni -trier.de/db/conf/date/",
  "7 EuroSys  European Conference on Computer Systems  ACM  http://dblp.uni -trier.de/db/conf/eurosys/",
  "8 HOT CHIPS  Hot Chips: A Symposium on High Performance Chips  IEEE  http://www.hotchips.org/",
  "9 CLUSTER  IEEE International Conference on Cluster Computing  IEEE  https://dblp.uni -trier.de/db/conf/cluster/",
  "10 ICCD  International Conference on Computer Design  IEEE  http://dblp.uni -trier.de/db/conf/iccd/",
  "11 ICCAD  International Conference on Computer -Aided Design  IEEE/ACM  http://dblp.uni -trier.de/db/conf/iccad/",
  "12 ICDCS  International Conference on Distributed Computing Systems  IEEE  http://dblp.uni -trier.de/db/conf/icdcs/",
  "13 CODES+ISSS  International Conference on Hardware/Software Co -design andSystem Synthesis  ACM/ IEEE  http://dblp.uni -trier.de/db/conf/codes/index.html",
  "14 HiPEAC  International Conference on High Performance and EmbeddedArchitectures and Compilers  ACM  http://dblp.uni -trier.de/db/conf/hipeac/  ",
  "15 SIGMETRICS  International Conference on Measurement and Modeling ofComputer Systems  ACM  http://dblp.uni -trier.de/db/conf/sigmetrics/",
  "16 PACT  International Conference on Parallel Architectures andCompilation Techniques  IEEE/ACM  http://dblp.uni -trier.de/db/conf/IEEEpact/",
  "17 ICPP  International Conference on Parallel Processing   http://dblp.uni -trier.de/db/conf/icpp/",
  "18 ICS  International Conference on Supercomputing  ACM  http://dblp.uni -trier.de/db/conf/ics/",
  "19 VEE  International Conference on Virtual Execution Environments  ACM  http://dblp.uni -trier.de/db/conf/vee/",
  "20 IPDPS  International Parallel & Distributed Processing Symposium  IEEE  http://dblp.uni -trier.de/db/conf/ipps/",
  "21 Performance  International Symposium on Computer Performance Modeling,Measurements and Evaluation  ACM  http://dblp.uni -trier.de/db/conf/performance/",
  "22 HPDC  International Symposium on High Performance DistributedComputing  IEEE  http://dblp.uni -trier.de/db/conf/hpdc/",
  "23 ITC  International Test Conference  IEEE  http://dblp.uni -trier.de/db/conf/itc/",
  "24 LISA  Large Installation System Administration Conference  USENIX  http://dblp.uni -trier.de/db/conf/lisa/",
  "25 MSST  Mass Storage Systems and Technologies  IEEE  http://dblp.uni -trier.de/db/conf/mss/",
  "26 RTAS  Real -Time and Embedded Technology and ApplicationsSymposium  IEEE  http://dblp.uni -trier.de/db/conf/rtas/",
  "1 SenSys  ACM Conference on Embedded Networked Sensor Systems  ACM  http://dblp.uni -trier.de/db/conf/sensys/",
  "2 CoNEXT  ACM International Conference on Emerging NetworkingExperiments and Technologies  ACM  http://dblp.uni -trier.de/db/conf/conext/",
  "3 SECON  IEEE Communications Society Conference on Sensor and Ad HocCommunications and Networks  IEEE  http://dblp.uni -trier.de/db/conf/secon/",
  "4 IPSN  International Conference on Information Processi ng in SensorNetworks  IEEE/ACM  http://dblp.uni -trier.de/db/conf/ipsn/",
  "5 MobiSys  International Conference on Mobile Systems, Applications, andServices  ACM  http://dblp.uni -trier.de/db/conf/mobisys/",
  "6 ICNP  International Conference on Network Protocols  IEEE  http://dblp.uni -trier.de/db/conf/icnp/",
  "7 MobiHoc  International Symposium on Mobile Ad Hoc Networking andComputing  ACM/IEEE  http://dblp.uni -trier.de/db/conf/mobihoc/",
  "8 NOSSDAV  International Workshop on Network and Operating System Supportfor Digital Audio and Video  ACM  http://dblp.uni -trier.de/db/conf/nossdav/",
  "9 IWQoS  International Workshop on Quality of Service  IEEE  http://dblp.uni -trier.de/db/conf/iwqos/",
  "10 IMC  Internet Measurement Conference  ACM/USENIX  http://dblp.uni -trier.de/db/conf/imc/  ",
  "1 ACSAC  Annual Computer Security Applications Conference  IEEE  http://dblp.uni -trier.de/db/conf/acsac/",
  "2 ASIACRYPT  Annual International Conference on the Theory andApplication of Cryptology and Information Security  Springer  http://dblp.uni -trier.de/db/conf/asiacrypt/",
  "3 ESORICS  European Symposium on Research in Computer Security  Springer  http://dblp.uni -trier.de/db/conf/esorics/",
  "4 FSE  Fast Software Encryption  Springer  http://dblp.uni -trier.de/db/conf/fse/",
  "5 CSFW  IEEE Computer Security Foundations Workshop  IEEE  http://dblp.uni -trier.de/db/conf/csfw/",
  "6 SRDS  IEEE International Symposium on Reliable DistributedSystems  IEEE  http://dblp.uni -trier.de/db/conf/srds/",
  "7 CHES  International Conference on Cryptographic Hardware andEmbedded Systems  Springer  http://dblp.uni -trier.de/db/conf/ches/",
  "8 DSN  International Conference on Dependable Systems andNetworks  IEEE/IFIP  http://dblp.uni -trier.de/db/conf/dsn/",
  "9 RAID  International Symposium on Recent Advances in IntrusionDetection  Springer  http://dblp.uni -trier.de/db/conf/raid/",
  "10 PKC  International Workshop on Practice and Theory in PublicKey Cryptography  Springer  http://dblp.uni -trier.de/db/conf/pkc/",
  "11 NDSS  ISOC Network and Distributed System SecuritySymposium  ISOC  http://dblp.uni -trier.de/db/conf/ndss/",
  "12 TCC  Theory of Cryptography Conference  Springer  http://dblp.uni -trier.de/db/conf/tcc/",
  "1 ECOOP  European Conference on Object -Oriented Programming  AITO  http://dblp.uni -trier.de/db/conf/ecoop/",
  "2 ETAPS  European Joint Conferences on Theory and Practice ofSoftware  Springer  http://dblp.uni -trier.de/db/conf/etaps/",
  "3 ICPC  IEEE International Conference on ProgramComprehension  IEEE  http://dblp.uni -trier.de/db/conf/iwpc/",
  "4 RE  IEEE International Requirement Engineering Conference  IEEE  http://db lp.uni -trier.de/db/conf/re/",
  "5 CAiSE  International Conference on Advanced InformationSystems Engineering  Springer  http://dblp.uni -trier.de/db/conf/caise/",
  "6 ICFP  International Conference on Function Programming  ACM  http://dblp.uni -trier.de/db/conf/icfp/",
  "7 LCTES  International Conference on Languages, Compilers andTools for Embedded Systems  ACM  http://dblp.uni -trier.de/db/conf/lctrts/",
  "8 MoDELS  International Conference on Model Driven EngineeringLanguages and Systems  ACM, IEEE  http://dblp.uni -trier.de/db/conf/models/",
  "9 CP International Conference on Principles and Practice ofConstraint Programming  Springer  http://dblp.uni -trier.de/db/conf/cp/",
  "10 ICSOC  International Conference on Service Oriented Computing  Springer  http://dblp.uni -trier.de/db/conf/icsoc/  ",
  "11 SANER  International Conference on Software Analysis, Evolution,and Reengineering  IEEE  http://dblp.uni -trier.de/db/conf/wcre/",
  "12 ICSME  International Conference on Software Maintenance andEvolution  IEEE  http://dblp.uni -trier.de/db/conf/icsm/",
  "13 VMCAI  International Conference on Verification, Model Checking,and Abstract Interpretation  Springer  http://dblp.uni -trier.de/db/conf/vmcai/",
  "14 ICWS  International Conference on Web Services \uff08ResearchTrack\uff09 IEEE  http://dblp.uni -trier.de/db/conf/icws/",
  "15 Middleware  International Middleware Conference  ACM/IFIP/USENIX  http://dblp.uni -trier.de/db/conf/middleware/",
  "16 SAS  International Static Analysis Symposium  Springer  http://dblp.uni -trier.de/db/conf/sas/",
  "17 ESEM  International Symposium on Empirical SoftwareEngineering and Measurement  ACM/IEEE  http://dblp.uni -trier.de/db/conf/esem/",
  "18 FM  International Symposium on Formal Methods  FME  http://dblp.uni -trier.de/db/conf/fm/",
  "19 ISSRE  International Symposium on Software ReliabilityEngineering  IEEE  http://dblp.uni -trier.de/db/conf/issre/",
  "20 HotOS  USENIX Workshop on Hot Topics in Operating Systems  USENIX  http://dblp.uni -trier.de/db/conf/hotos/  ",
  "1 CIKM  ACM International Conference on Informationand Knowledge Management  ACM  http://dblp.uni -trier.de/db/conf/cikm/",
  "2 WSDM  ACM International Conference on Web Searchand Data Mining  ACM  http://dblp.uni -trier.de/db/conf/wsdm/",
  "3 PODS  ACM Symposium on Principles of DatabaseSystems  ACM  http://dblp.uni -trier.de/db/conf/pods/",
  "4 DASFAA  Database Systems for Advanced Applications  Springer  http://dblp.uni -trier.de/db/conf/dasfaa/",
  "5 ECML -PKDD  European Conference on Machine Learningand Principles and Practice of KnowledgeDiscovery in Databases  Springer  http://dblp.uni -trier.de/db/conf/pkdd/",
  "5 ECML -PKDD  European Conference on Machine Learningand Principles and Practice of KnowledgeDiscovery in Databases  Springer  http://dblp.uni -trier.de/db/conf/ecml/",
  "6 ISWC  IEEE International Semantic Web Conference  IEE E http://dblp.uni -trier.de/db/conf/semweb/",
  "7 ICDM  International Conference on Data Mining  IEEE  http://dblp.uni -trier.de/db/conf/icdm/",
  "8 ICDT  International Conference on Database Theory  Springer  http://dblp.uni -trier.de/db/conf/icdt/",
  "9 EDBT  International Conference on Extending DBTechnology  Springer  http://dblp.uni -trier.de/db/conf/edbt/",
  "10 CIDR  International Conference on Innovative DataSystems Research  Online Proceeding  http://dblp.uni -trier.de/db/conf/cidr/",
  "11 SDM  SIAM International Conference on DataMining  SIAM  http://dblp.uni -trier.de/db/conf/sdm/",
  "1 SoCG  ACM Symposium on Computational Geometry  ACM  http://dblp.uni -trier.de/db/conf/compgeom/",
  "2 ESA  European Symposium on Algorithms  Springer  http://dblp.uni -trier.de/db/conf/esa/",
  "3 CCC  IEEE Conference on Computational Complexity  IEEE  http://dblp.uni -trier.de/db/conf/coco/",
  "4 ICALP  International Colloquium on Automata, Languages andProgramming  Springer  http://dblp.uni -trier.de/db/conf/icalp/",
  "5 CADE/IJCAR  International Conference on AutomatedDeduction/International Joint Conference onAutomated Reasoning  Springer  http://dblp.uni -trier.de/db/conf/cade/",
  "6 CONCUR  International Conference on Concurrency Theory  Springer  http://dblp.uni -trier.de/db/conf/concur/",
  "7 HSCC  International Conference on Hybrid Systems:Computation and Control  Springer and ACM  http://dblp.uni -trier.de/db/conf/hybrid/",
  "8 SAT  Theory and Applications of Satisfiability Testing  Springer  http://dblp.uni -trier.de/db/conf/sat/",
  "1 ICMR  ACM SIGMM International Conference onMultimedia Retrieval  ACM  http://dblp.uni -trier.de/db/conf/mir/",
  "2 SI3D  ACM Symposium on Interactive 3D Graphics  ACM  http://dblp.uni -trier.de/db/conf/si3d/",
  "3 SCA  ACM/Eurographics Symposium on ComputerAnimation  ACM  http://dblp.uni -trier.de/db/conf/sca/index.html",
  "4 DCC  Data Compression Conference  IEEE  http://dblp.uni -trier.de/db/conf/dcc/",
  "5 EG  Eurographics  Wiley/ Blackwell  http://dblp.uni -trier.de/db/conf/eurographics/",
  "6 EuroVis  Eurographics  Conference on Visualization  ACM  http://dblp.uni -trier.de/db/conf/vissym/",
  "7 SGP  Eurographics Symposium on GeometryProcessing   Wiley/Blackwell  http://dblp.uni -trier.de/db/conf/sgp/",
  "8 EGSR  Eurographics Symposium on Rendering  Wiley/Blackwell  http://dblp.uni -trier.de/db/conf/rt/",
  "9 ICASSP  IEEE International Conference on Acoustics,Speech and SP  IEEE  http://dblp.uni -trier.de/db/conf/icassp/",
  "10 ICME  IEEE International Conference onMultimedia & Expo  IEEE  http://dblp.uni -trier.de/db/conf/icmcs/",
  "11 ISMAR  International Symposium on Mixed andAugmented Reality  IEEE/ACM  http://dblp.uni -trier.de/db/conf/ismar/",
  "12 PG  Pacific Graphics: The Pacific Conference on Computer Graphics and Applications  Wiley/Blackwell  http://dblp.uni -trier.de/db/conf/pg/index.html  ",
  "13 SPM  Symposium on Solid and Physical Modeling  SMA/Elsevier  http://dblp.uni -trier.de/db/conf/sma/  ",
  "1 COLT  Annual Conference onComputational Learning Theory  Springer  http://dblp.uni -trier.de/db/conf/colt/",
  "2 EMNLP  Conference on EmpiricalMethods in Natural LanguageProcessing  ACL  http://dblp.uni -trier.de/db/conf/emnlp/",
  "3 ECAI  European Conference onArtificial Intelligence  IOS Press  http://dblp.uni -trier.de/db/conf/ecai/",
  "4 ECCV  European Conference onComputer Vision  Springer  http://dblp.uni -trier.de/db/conf/eccv/",
  "5 ICRA  IEEE International Conference onRobotics and Automation  IEEE  http://dblp.uni -trier.de/db/conf/icra/",
  "6 ICAPS  International Conference onAutomated Planning andScheduling  AAAI  http://dblp.uni -trier.de/db/conf/aips/",
  "7 ICCBR  International Conference onCase -Based Reasoning andDevelopment  Springer  http://dblp.uni -trier.de/db/conf/iccbr/",
  "8 COLING  International Conference onComputational Linguistics  ACM  http://dblp.uni -trier.de/db/conf/coling/  ",
  "9 KR  International Conference onPrinciples of KnowledgeRepresentation and Reasoning  Morgan Kaufmann  http://dblp.uni -trier.de/db/conf/kr/",
  "10 UAI  International Conference onUncertainty in ArtificialIntelligence  AUAI  http://dblp.uni -trier.de/db/conf/uai/",
  "11 AAMAS  International Joint Conference onAutonomous Agents and Multi -agent Systems  Springer  http://dblp.uni -trier.de/db/conf/atal/index.html",
  "12 PPSN  Parallel Problem Solving from Nature  Springer  http://dblp.uni -trier.de/db/conf/ppsn/  ",
  "1 GROUP  ACM Conference on Supporting Group Work  ACM  http://dblp.uni -trier.de/db/conf/group/",
  "2 IUI  ACM International Conference on Intelligent UserInterfaces  ACM  http://dblp.uni -trier.de/db/conf/iui/",
  "3 ITS  ACM International Conference on InteractiveTabletops and Surfaces  ACM  http://dblp.uni -trier.de/db/conf/tabletop/",
  "4 UIST  ACM Symposium on User Interface Software andTechnology  ACM  http://dblp.uni -trier.de/db/conf/uist/",
  "5 ECSCW  European Conference on Computer SupportedCooperative Work  Springer  http://dblp.uni -trier.de/db/conf/ecscw/",
  "6 PERCOM  IEEE International Conference on PervasiveComputing and Communications  IEEE  http://dblp.uni -trier.de/db/conf/percom/",
  "7 MobileHCI  International Conference on Human Computer Interaction with Mobile Devices and Services  ACM  http://dblp.uni -trier.de/db/conf/mhci/  ",
  "1 CogSci  Cognitive Science Society Annual Conference  Psychology Press  https://dblp.uni -trier.de/db/conf/cogsci/",
  "2 BIBM  IEEE International Conference on Bioinformaticsand Biomedicine  IEEE  http://dblp.uni -trier.de/db/conf/bibm/",
  "3 EMSOFT  International Conference on Embedded Software  ACM/IEEE/IFIP  http://dblp.uni -trier.de/db/conf/emsoft/",
  "4 ISMB  International conference on Intelligent Systems forMolecular Biology  Oxford Journals  http://www.iscb.org/about -ismb",
  "5 RECOMB  International Conference on Research inComputational Molecular Biology  Springer  http://dblp.uni -trier.de/db/conf/recomb/"
]


import abc
import asyncio
import logging
from itertools import combinations
from typing import Optional, Iterable

from dblp_crawler import download_person, DBLPPerson, Publication, download_journal_list, JournalList

logger = logging.getLogger("graph")


class Graph(metaclass=abc.ABCMeta):
    def __init__(self, pid_list: list[str], journal_list: list[str]) -> None:
        self.persons: dict[str, Optional[DBLPPerson]] = {pid: None for pid in pid_list}
        self.checked: set[str] = set()
        self.publications: dict[str, Publication] = {}
        self.init_journals = journal_list
        self.journals_inited = False

    async def init_persons_from_journals(self) -> None:
        author_count = 0
        publication_count = 0
        for jid in self.init_journals:
            data = await download_journal_list(jid)
            if data is None:
                return
            jl = JournalList(data)
            async for journal in jl.journals():
                for publication in self.filter_publications_at_crawler(journal.publications()):
                    if publication.key() in self.publications:
                        continue  # 已经遍历过的文章不再重复
                    self.publications[publication.key()] = publication  # 记录下这个文章已遍历
                    publication_count += 1
                    logger.debug(str(publication))
                    for author in publication.authors():
                        if author.pid() not in self.persons:  # 如果作者不存在
                            self.persons[author.pid()] = None  # 就加入作者
                            author_count += 1
        logger.info("%d initial authors added from %d publications" % (author_count, publication_count))

    @abc.abstractmethod
    def filter_publications_at_crawler(self, publications: Iterable[Publication]) -> Iterable[Publication]:
        """在收集信息时过滤`Publication`，不会对被此方法过滤掉的`Publication`进行信息收集"""
        for publication in publications:
            yield publication

    @abc.abstractmethod
    def filter_publications_at_output(self, publications: Iterable[Publication]) -> Iterable[Publication]:
        """在输出时过滤`Publication`，被过滤掉的`Publication`将不会出现在输出中"""
        for publication in publications:
            yield publication

    async def download_person(self, pid: str) -> None:
        data = await download_person(pid)
        if data is None:
            return
        self.persons[pid] = DBLPPerson(data)

    async def bfs_once(self) -> tuple[int, int]:
        if not self.journals_inited:
            await self.init_persons_from_journals()

        tasks = []
        init_author_count = 0
        for pid, person in list(self.persons.items()):
            if person is None:
                tasks.append(asyncio.create_task(self.download_person(pid)))
                init_author_count += 1
        logger.info("Initializing %d authors" % init_author_count)
        await asyncio.gather(*tasks)

        tasks = []
        total_author_count = 0
        total_publication_count = 0
        for pid, person in list(self.persons.items()):
            author_count = 0
            publication_count = 0
            if person is None:
                tasks.append(asyncio.create_task(self.download_person(pid)))
                continue  # 还没下载的节点先下载
            if pid in self.checked:
                continue  # 已经遍历过的节点不再重复
            self.checked.add(pid)  # 记录下这个节点已遍历
            for publication in self.filter_publications_at_crawler(person.publications()):
                if publication.key() in self.publications:
                    continue  # 已经遍历过的文章不再重复
                self.publications[publication.key()] = publication  # 记录下这个文章已遍历
                publication_count += 1
                logger.debug(str(publication))
                for author in publication.authors():
                    if author.pid() not in self.persons:  # 如果作者不存在
                        tasks.append(asyncio.create_task(self.download_person(author.pid())))  # 就获取作者
                        self.persons[author.pid()] = None  # 并记录之
                        author_count += 1
            logger.info("there are %d authors in %d %s's publications" % (author_count, publication_count, pid))
            total_author_count += author_count
            total_publication_count += publication_count
        logger.info("there are %d authors in %d publications" % (total_author_count, total_publication_count))
        await asyncio.gather(*tasks)
        logger.info("%d authors added from %d publications" % (total_author_count, total_publication_count))
        remain_none = 0
        for person in self.persons.values():
            if person is None:
                remain_none += 1
        logger.info("There are %d authors need to be downloaded in next loop" % remain_none)
        return remain_none, total_author_count

    @abc.abstractmethod
    def summarize_person(self, a: str, person: Optional[DBLPPerson]) -> None:  # 构建summary
        """你想要如何Summary一个`Person`数据？实现此方法"""
        pass

    @abc.abstractmethod
    def summarize_publication(self, a: str, b: str, publication: Publication) -> None:  # 构建summary
        """你想要如何Summary一个`Publication`数据？实现此方法"""
        pass

    def summarize(self) -> None:
        """执行`summarize_person`和`summarize_publication`指定的Summary过程"""
        summarized_persons = set()
        for publication in self.filter_publications_at_output(list(self.publications.values())):  # 遍历所有文章
            authors_pid = {author.pid() for author in publication.authors()}  # 获取作者列表
            for a, b in combinations(authors_pid, 2):  # 列表中的作者两两之间形成边
                if a == b or self.persons[a] is None or self.persons[b] is None:
                    continue
                if a not in summarized_persons:
                    self.summarize_person(a, person=self.persons[a])  # 把作者信息加进图里
                    summarized_persons.add(a)
                if b not in summarized_persons:
                    self.summarize_person(b, person=self.persons[b])  # 把作者信息加进图里
                    summarized_persons.add(b)
                self.summarize_publication(a, b, publication=publication)  # 把边加进图里



data = [
  "1 JETC  ACM Journal on Emerging Technologies inComputing Systems  ACM  http://dblp.uni -trier.de/db/journals/jetc/",
  "2  Concurrency and Computation: Practice andExperience  Wiley  http://dblp.uni -trier.de/db/journals/concurrency/",
  "3 DC  Distributed Computing  Springer  http://dblp.uni -trier.de/db/journals/dc/",
  "4 FGCS  Future Generation Computer Systems  Elsevier  http://dblp.uni -trier.de/db/journals/fgcs/",
  "5 TCC  IEEE Transactions on Cloud Computing  IEEE  https://dblp.uni -trier.de/db/journals/tcc/",
  "6 Integration  Integration, the VLSI Journal  Elsevier  http://dblp.uni -trier.de/db/journals/integration/",
  "7 JET TA  Journal of Electronic Testing -Theory andApplications  Springer  http://link.springer.com/journal/10836",
  "8 JGC  Journal of Grid computing  Springer  https://dblp.uni -trier.de/db/journals/grid/",
  "9 MICPRO  Microprocessors and Microsystems: EmbeddedHardware Design  Elsevier  http://dblp.uni -trier.de/db/journals/mam/",
  "10 RTS  Real -Time Systems  Springer  http://dblp.uni -trier.de/db/journals/rts/",
  "11 TJSC  The Journal of Supercomputing  Springer  http://dblp.uni -trier.de/db/journals/tjs/  ",
  "1  Ad Hoc Networks  Elsevier  http://dblp.uni -trier.de/db/journals/adhoc/",
  "2 CC  Computer Communications  Elsevier  http://dblp.uni -trier.de/db/journals/comcom/",
  "3 TNSM  IEEE Transactions on Network and ServiceManagement  IEEE  http://dblp.uni -trier.de/db/journals/tnsm/",
  "4  IET Communications  IET  http://dblp.uni -trier.de/db/journals/iet -com/",
  "5 JNCA  Journal of Network and Computer Applications  Elsevier  http://dblp.uni -trier.de/db/journals/jnca/",
  "6 MONET  Mobile Networks & Applications  Springer  http://dblp.uni -trier.de/db/journals/monet/",
  "7  Networks  Wiley  http://dblp.uni -trier.de/db/journals/networks/",
  "8 PPNA  Peer -to-Peer Networking and Applications  Springer  http://dblp.uni -trier.de/db/journals/ppna/",
  "9 WCMC  Wireless Communications & Mobile Computing  Wiley.  http://dblp.uni -trier.de/db/journals/wicomm/",
  "10  Wireless Networks  Springer  http://dblp.uni -trier.de/db/journals/winet/  ",
  "1 CLSR  Computer Law and Security Review  Elsevier  http://www.journals.elsevier.com/computer -law -and -security -review/",
  "2   EURASIP Journal on Information Security  Springer  http://dblp.uni -trier.de/db/journals/ejisec/",
  "3   IET Information Security  IET  http://dblp.uni -trier.de/db/journals/iet -ifs/",
  "4 IMCS  Information Management & Computer Security  Emerald  http://dblp.uni -trier.de/db/journals/imcs/",
  "5 IJICS  International Journal of Information and Computer Security  Inderscience  http://dblp.uni -trier.de/db/journals/ijics/",
  "6 IJISP  International Journal of Information Security and Privacy  Idea Group Inc  http://dblp.uni -trier.de/db/journals/ijisp/",
  "7 JISA  Journal of Information Security and Application  Elsevier  https://dblp.uni -trier.de/db/journals/istr/",
  "8 SCN  Security and Communication Networks  Wiley  http://dblp.uni -trier.de/db/journals/scn/  ",
  "1 CL  Computer Languages, Systems and Structures  Elsevier  http://dblp.uni -trier.de/db/journals/cl/index.html",
  "2 IJSEKE  International Journal on Software Engineering andKnowledge Engineering  World Scientific  http://dblp.uni -trier.de/db/journals/ijseke/index.html",
  "3 STTT  International Journal on Software Tools for TechnologyTransfer  Springer  http://dblp.uni -trier.de/db/journals/sttt/",
  "4 JLAP  Journal of Logic and Algebraic Programming  Elsevier  http://dblp.uni -trier.de/db/journals/jlp/jlap.html",
  "5 JWE  Journal of Web Engineering  Rinton Press  http://dblp.uni -trier.de/db/journals/jwe/",
  "6 SOCA  Service Oriented Computing and Applications  Springer  http://dblp.uni -trier.de/db/journals/soca/",
  "7 SQJ  Software Quality Journal  Springer   http://dblp.uni -trier.de/db/journals/sqj/",
  "8 TPLP  Theory and Practice of Logic Programming  Cambridge UniversityPress  http://dblp.uni -trier.de/db/journals/tplp/",
  "1 DPD  Distributed and Parallel Databases  Springer  http://dblp.uni -trier.de/db/journals/dpd/",
  "2 I&M  Information and Management  Elsevier  http://dblp.uni -trier.de/db/journals/iam/",
  "3 IPL  Information Processing Letters  Elsevier  http://dblp.uni -trier.de/db/journals/ipl/",
  "4 IR Information Retrieval Journal  Springer  http://dblp.uni -trier.de/db/journals/ir/",
  "5 IJCIS  International Journal of Cooperative Information Systems  World Scientific  http://dblp.uni -trier.de/db/journals/ijcis/",
  "6 IJGIS  International Journal of Geographical InformationScience  Taylor & Francis  http://dblp.uni -trier.de/db/journals/gis/",
  "7 IJIS  International Journal of Intelligent Systems  Wiley  http://dblp.uni -trier.de/db/journals/ijis/",
  "8 IJKM  International Journal of Knowledge Management  IGI  http://dblp.uni -trier.de/db/journals/ijkm/",
  "9 IJSWIS  International Journal on Semantic Web and InformationSystems  IGI  http://dblp.uni -trier.de/db/journals/ijswis/",
  "10 JCIS  Journal of Computer Information Systems  IACIS  http://dblp.uni -trier.de/db/journals/jcis/",
  "11 JDM  Journal of Database Management  IGI -Global  http://dblp.uni -trier.de/db/journals/jdm/",
  "12 JGITM  Journal of Global Information Technology Management  Ivy LeaguePublishing  http://www.tandfonline.com/loi/ugit20#.Vnv35pN97rI",
  "13 JIIS  Journal of Intelligent Information Systems  Springer  http://dblp.uni -trier.de/db/journals/jiis/",
  "14 JSIS  Journal of Strategic Information Systems  Elsevier  http://dblp.uni -trier.de/db/journals/jsis/  ",
  "1 ACTA  Acta Informatica  Springer  http://dblp.uni -trier.de/db/journals/acta/",
  "2 APAL  Annals of Pure and Applied Logic  Elsevier  http://dblp.uni -trier.de/db/journals/apal/",
  "3 DAM  Discrete Applied Mathematics  Elsevier  http://dblp.uni -trier.de/db/journals/dam/",
  "4 FUIN  Fundamenta Informaticae  IOS Press  http://dblp.uni -trier.de/db/journals/fuin/",
  "5 LISP  Higher -Order and Symbolic Computation  Springer  http://dblp.uni -trier.de/db/journals/lisp/",
  "6 IPL  Information Processing Letters  Elsevier  http://dblp.uni -trier.de/db/journals/ipl/",
  "7 JCOMPLEXITY  Journal of Complexity  Elsevier  http://dblp.uni -trier.de/db/journals/jc/",
  "8 LOGCOM  Journal of Logic and Computation  Oxford University Press  http://dblp.uni -trier.de/db/journals/logcom/",
  "9 JSL  Journal of Symbolic Logic  Association for Symbolic Logic  http://dblp.uni -trier.de/db/journals/jsyml/",
  "10 LMCS  Logical Methods in Computer Science  LMCS  http://dblp.uni -trier.de/db/journals/lmcs/",
  "11 SIDMA  SIAM Journal on Discrete Mathematics  SIAM  http://dblp.uni -trier.de/db/journals/siamdm/",
  "12   Theory of Computing Systems  Springer  http://dblp.uni -trier.de/db/journals/mst/",
  "1 CGTA  Computational Geometry: Theory andApplications  Elsevier  http://dblp.uni -trier.de/db/journals/comgeo/",
  "2 CAVW  Computer Animation and Virtual Worlds  Wiley  http://onlinelibrary.wiley.com/journal/10.1002/(ISSN)1546-427X",
  "3 C&G  Computers & Graphics  Elsevier  http://dblp.uni -trier.de/db/journals/cg/",
  "4 DCG  Discrete & Computational Geometry  Springer  http://dblp.uni -trier.de/db/journals/dcg/",
  "5 SPL  IEEE Signal Processing Letters  IEEE  http://dblp.uni -trier.de/db/journals/spl/",
  "6 IET -IPR  IET Image Processing  IET  http://dblp.uni -trier.de/db/journals/iet -ipr/",
  "7 JVCIR  Journal of Visual Communication and ImageRepresentation  Elsevier  http://dblp.uni -trier.de/db/journals/jvcir/",
  "8 MS  Multimedia Systems  Springer  http://dblp.uni -trier.de/db/journals/mms/",
  "9 MTA  Multimedia Tools and Applications  Springer  http://dblp.uni -trier.de/db/journals/mta/",
  "10   Signal Processing  Elsevier  http://dblp.uni -trier.de/db/journals/sigpro/",
  "11 SPIC  Signal Processing: Image Communication  Elsevier  http://dblp.uni -trier.de/db/journals/spic/",
  "12 TVC  The Visual Computer  Springer  http://dblp.uni -trier.de/db/journals/vc/",
  "1 TALLIP  ACM Transactions on Asian and Low -Resource  Language Information Processing  ACM  http://dblp.uni -trier.de/db/journals/talip/",
  "2   Applied Intelligence  Springer  http://dblp.uni -trier.de/db/journals/apin/",
  "3 AIM  Artificial Intelligence in Medicine  Elsevier  http://dblp.uni -trier.de/db/journals/artmed/",
  "4   Artificial Life  MIT Press  http://dblp.uni -trier.de/db/journals/alife/",
  "5   Computational Intelligence  Wiley  http://dblp.uni -trier.de/db/journals/ci/",
  "6   Computer Speech and Language  Elsevier  http://dblp.uni -trier.de/db/journals/csl/",
  "7   Connection Science  Taylor & Francis  http://dblp.uni -trier.de/db/journals/connection/",
  "8 DSS  Decision Support Systems  Elsevier  http://dblp.uni -trier.de/db/journals/dss/",
  "9 EAAI  Engineering Applications of ArtificialIntelligence  Elsevier  http://dblp.uni -trier.de/db/journals/eaai/",
  "10   Expert Systems  Blackwell/Wiley  http://dblp.uni -trier.de/db/journals/es/",
  "11 ESWA  Expert Systems with Applications  Elsevier  http://dblp.uni -trier.de/db/journals/eswa/",
  "12   Fuzzy Sets and Systems  Elsevier   http://dblp.uni -trier.de/db/journals/fss/",
  "13 TG  IEEE Transactions on Games  IEEE  http://dblp.uni -trier.de/db/journals/tciaig/",
  "14 IET -CVI  IET Computer Vision  IET  http://digital -library.theiet.org/content/journals/iet -cvi",
  "15   IET Signal Processing  IET  * http://digital -library.theiet.org/content/journals/iet -spr",
  "16 IVC  Image and Vision Computing  Elsevier  http://dblp.uni -trier.de/db/journals/ivc/  ",
  "17 IDA  Intelligent Data Analysis  IOS Press  http://dblp.uni -trier.de/db/journals/ida/",
  "18 IJCIA  International Journal of ComputationalIntelligence and Applications  World Scientific  http://dblp.uni -trier.de/db/journals/ijcia/",
  "19 IJIS  International Journal of Intelligent Systems  Wiley  http://dblp.uni -trier.de/db/journals/ijis/",
  "20 IJNS  International Journal of Neural Systems  World Scientific  http://dblp.uni -trier.de/db/journals/ijns/",
  "21 IJPRAI  International Journal of Pattern Recognitionand Artificial Intelligence  World Scientific  http://dblp.uni -trier.de/db/journals/ijprai/",
  "22 IJUFKS  International Journal of Uncertainty,Fuzziness and Knowledge -Based System  World Scientific  https://dblp.uni -trier.de/db/journals/ijufks/",
  "23 IJDAR  International Journal on Document Analysisand Recognition  Springer  http://dblp.uni -trier.de/db/journals/ijdar/",
  "24 JETAI  Journal of Experimental and TheoreticalArtificial Intelligence  Taylor & Francis  http://dblp.uni -trier.de/db/journals/jetai/",
  "25 KBS  Knowledge -Based Systems  Elsevier  http://dblp.uni -trier.de/db/journals/kbs/",
  "26   Machine Translation  Springer  http://dblp.uni -trier.de/db/journals/mt/",
  "27   Machine Vision and Applications  Springer  http://dblp.uni -trier.de/db/journals/mva/",
  "28   Natural Computing  Springer  http://dblp.uni -trier.de/db/journals/nc/",
  "29 NLE  Natural Language Engineering  CambridgeUniversity Press  http://dblp.uni -trier.de/db/journals/nle/",
  "30 NCA  Neural Computing & Applications  Springer  http://dblp.uni -trier.de/db/journals/nca/",
  "31 NPL  Neural Processing Letters  Springer  http://dblp.uni -trier.de/db/journals/npl/",
  "32   Neurocomputing  Elsevier  http://dblp.uni -trier.de/db/journals/ijon/  ",
  "33 PAA  Pattern Analysis and Applications  Springer  http://dblp.uni -trier.de/db/journals/paa/",
  "34 PRL  Pattern Recognition Letters  Elsevier  http://dblp.uni -trier.de/db/journals/prl/",
  "35   Soft Computing  Springer  http://dblp.uni -trier.de/db/journals/soco/",
  "36 WI  Web Intelligence  IOS Press  http://dblp.uni -trier.de/db/journals/wias/",
  "1 BIT  Behaviour & Information Technology  Taylor & Francis  http://dblp.uni -trier.de/db/journals/behaviourIT/",
  "2 PUC  Personal and Ubiquitous Computing  Springer  http://dblp.uni -trier.de/db/journals/puc/",
  "3 PMC  Pervasive and Mobile Computing  Elsevier  http://dblp.uni -trier.de/db/journals/percom/",
  "1  BMC Bioinformatics  BioMed Central  http://dblp.uni -trier.de/db/journals/bmcbi/",
  "2  Cybernetics and Systems  Taylor & Francis  http://dblp.uni -trier.de/db/journals/cas/",
  "3 FCS  Frontiers of Computer Science  Higher EducationPress  http://dblp.uni -trier.de/db/journals/fcsc/",
  "4  IEEE Geoscience and Remote SensingLetters  IEEE  http://dblp.uni -trier.de/db/journals/lgrs/",
  "5 JBHI  IEEE Journal of Biomedical and HealthInformatics  IEEE  http://dblp.uni -trier.de/db/journals/titb/",
  "6 TBD  IEEE Transactions on Big Data  IEEE  https://dblp.uni -trier.de/db/journals/tbd/",
  "7  IET Intelligent Transport Systems  IET  http://digital -library.theiet.org/content/journals/iet -its",
  "8 JBI  Journal of Biomedical Informatics  Elsevier  https://dblp.uni -trier.de/db/journals/jbi/",
  "9  Medical Image Analysis  Elsevier  http://dblp.uni -trier.de/db/journals/mia/  ",
  "1 CF ACM International Conference on Computing Frontiers  ACM  http://dblp.uni -trier.de/db/conf/cf",
  "2 SYSTOR  ACM International Systems and Storage Conference  ACM  http://dblp.uni -trier.de/db/conf/systor/index.html",
  "3 NOCS  ACM/IEEE International Symposium on Networks -on-Chip  ACM/IEEE  http://dblp.uni -trier.de/db/conf/nocs",
  "4 ASAP  Application -Specific Systems, Architectures, and Processors  IEEE  http://dblp.uni -trier.de/db/conf/asap",
  "5 ASP -DAC  Asia and South Pacific Design Automation Conference  ACM/IEEE  http://dblp.uni -trier.de/db/conf/aspdac",
  "6 Euro -Par  European Conference on Parallel and Distributed Computing  Springer  http://dblp.uni -trier.de/db/conf/europar/",
  "7 ETS  European Test Symposium  IEEE  http://dblp.uni -trier.de/db/conf/ets/",
  "8 FPL  Field Programmable Logic and Applications  IEEE  http://dblp.uni -trier.de/db/conf/fpl/",
  "9 FCCM  Field -Programmable Custom Computing Machines  IEEE  http://dblp.uni -trier.de/db/conf/fccm/",
  "10 GLSVLSI  Great Lakes Symposium on VLSI  ACM/IEEE  http://dblp.uni -trier.de/db/conf/glvlsi/",
  "11 ATS  IEEE Asian Test Symposium  IEEE  http://dblp.uni -trier.de/db/conf/ats/",
  "12 HPCC  IEEE International Conference on High PerformanceComputing and Communications  IEEE  http://dblp.uni -trier.de/db/conf/hpcc/",
  "13 HiPC IEEE International Conference on High PerformanceComputing, Data and Analytics  IEEE/ ACM  http://dblp.uni -trier.de/db/conf/hipc/index.html",
  "14 MASCOTS  IEEE International Symposium on Modeling, Analysis, andSimulation of Computer and Telecommunication Systems  IEEE  http://dblp.uni -trier.de/db/conf/mascots/  ",
  "15 ISPA  IEEE International Symposium on Parallel and DistributedProcessing with Applications  IEEE  http://dblp.uni -trier.de/db/conf/ispa/",
  "16 CCGRID  IEEE/ACM International Symposium on Cluster, Cloud andGrid Computing  ACM/IEEE  http://dblp.uni -trier.de/db/conf/ccgrid/",
  "17 NPC  IFIP International Conference on Network and ParallelComputing  Springer  http://dblp.uni -trier.de/db/conf/npc/",
  "18 ICA3PP  International Conference on Algorithms and Architectures forParallel Processing  IEEE  http://dblp.uni -trier.de/db/conf/ica3pp/",
  "19 CASES  International Conference on Compilers, Architectures, andSynthesis for Embedded Systems  ACM  http://dblp.uni -trier.de/db/conf/cases/index.html",
  "20 FPT  International Conference on Field -Programmable Technology  IEEE  http://dblp.uni -trier.de/db/conf/fpt/",
  "21 ICPADS  International Conference on Parallel and Distributed Systems  IEEE  http://dblp.uni -trier.de/db/conf/icpads/",
  "22 ISCAS  International Symposium on Circuits and Systems  IEEE  http://dblp.uni -trier.de/db/conf/iscas/",
  "23 ISLPED  International Symposium on Low Power Electronics andDesign  ACM/IEEE  http://dblp.uni -trier.de/db/conf/islped/",
  "24 ISPD  International Symposium on Physical Design  ACM  http://dblp.uni -trier.de/db/conf/ispd/",
  "25 Hot I Symposium on High -Performance Interconnects  IEEE  http://dblp.uni -trier.de/db/conf/hoti/",
  "26 VTS  VLSI Test Symposium  IEEE  http://dblp.uni -trier.de/db/conf/vts/  ",
  "1 ANCS  Architectures for Networking and Communications Systems  ACM/IEEE http://dblp.uni -trier.de/db/conf/ancs/",
  "2 APNOMS  Asia -Pacific Network Operations and Management Symposium  IFIP/IEEE  http://dblp.uni -trier.de/db/conf/apnoms/",
  "3 FORTE  Formal Techniques for Networked and Distributed Systems  Springer  http://dblp.uni -trier.de/db/conf/forte/",
  "4 LCN  IEEE Conference on Local Computer Networks  IEEE  http://dblp.uni -trier.de/db/conf/lcn/",
  "5 GLOBECOM IEEE Global Communications Conference  IEEE  http://dblp.uni -trier.de/db/conf/globecom/",
  "6 ICC  IEEE International Conference on Communications  IEEE  http://dblp.uni -trier.de/db/conf/icc/",
  "7 ICCCN  IEEE International Conference on Computer Communications andNetworks  IEEE  http://dblp.uni -trier.de/db/conf/icccn/",
  "8 MASS  IEEE International Conference on Mobile Ad -hoc and Sensor Systems  IEEE  http://dblp.uni -trier.de/db/conf/mass/index.html",
  "9 P2P  IEEE International Conference on Peer -to-Peer Computing  IEEE  http://dblp.uni -trier.de/db/conf/p2p/",
  "10 IPCCC  IEEE International Performance Computing and CommunicationsConference  IEEE  http://dblp.uni -trier.de/db/conf/ipccc/",
  "11 WoWMoM  IEEE International Symposium on a World of Wireless , Mobile andMultimedia Networks  IEEE  http://dblp.uni -trier.de/db/conf/wowmom/",
  "12 ISCC  IEEE Symposium on Computers and Communications  IEEE  http://dblp.uni -trier.de/db/conf/iscc/",
  "13 WCNC  IEEE Wireless Communications & Networking Conference  IEEE  http://dblp.uni -trier.de/db/conf/wcnc/  ",
  "14 Networking  IFIP International Conferences on Networking  IFIP  http://dblp.uni -trier.de/db/conf/networking/index.html",
  "15 IM IFIP/IEEE International Symposium on Integrated NetworkManagement  IFIP/IEEE  http://dblp.uni -trier.de/db/conf/im/index.html",
  "16 MSN  International Conference on Mobile Ad -hoc and Sensor Networks  IEEE  https://dblp.uni -trier.de/db/conf/msn/",
  "17 MSWiM  International Conference on Modeling, Analysis and Simulation ofWireless and Mobile Systems  ACM  http://dblp.uni -trier.de/db/conf/mswim/",
  "18 WASA  International Conference on Wireless Algorithms, Systems, andApplications  Springer  http://dblp.uni -trier.de/db/conf/wasa/",
  "19 HotNets  The Workshop on Hot Topics in Networks  ACM  http://dblp.uni -trier.de/db/conf/hotnets/",
  "1 WiSec  ACM Conference on Security and Privacy in Wireless andMobile Networks  ACM  http://dblp.uni -trier.de/db/conf/wisec/",
  "2 SACMAT  ACM Symposium on Access Control Models andTechnologies  ACM  http://dblp.uni -trier.de/db/conf/sacmat/",
  "3 DRM  ACM Workshop on Digital Rights Management  ACM  http://dblp.uni -trier.de/db/conf/drm/",
  "4 IH&MMSec  ACM Workshop on Information Hiding and MultimediaSecurity  ACM  http://dblp.uni -trier.de/db/conf/ih/",
  "5 ACNS  Applied Cryptography and Network Security  Springer  http://dblp.uni -trier.de/db/conf/acns/",
  "6 AsiaCCS  Asia Conference on Computer and Communications Security  ACM  http://dblp.uni -trier.de/db/conf/ccs/",
  "7 ACISP  Australasia Conference on Information Security and Privacy  Springer  http://dblp.uni -trier.de/db/conf/acisp/",
  "8 CT -RSA  Cryptographer's Track at RSA Conference  Springer  http://dblp.uni -trier.de/db/conf/ctrsa/",
  "9 DIMVA  Detection of Intrusions and Malware & VulnerabilityAssessment   http://dblp.uni -trier.de/db/conf/dimva/",
  "10 DFRWS  Digital Forensic Research Workshop  Elsevier  http://dblp.uni -trier.de/db/conf/dfrws/",
  "11 FC Financial Cryptography and Data Security  Springer  http://dblp.uni -trier.de/db/conf/fc/",
  "12 TrustCom  IEEE International Conference on Trust, Security and Privacyin Computing and Communications  IEEE  http://dblp.uni -trier.de/db/conf/trustcom/  ",
  "13 SEC  IFIP International Information Security Conference  Springer  http://dblp.uni -trier.de/db/conf/sec/",
  "14 IFIP WG 11.9  IFIP WG 11.9 International Conference on Digital Forensics  Springer  *http://www.ifip119.org/Conferences/",
  "15 ISC  Information Security Conference  Springer  http://dblp.uni -trier.de/db/conf/isw/",
  "16 ICDF2C  International Conference on Digital Forensics & Cyber Crime  Springer  http://dblp.uni -trier.de/db/conf/icdf2c/",
  "17 ICICS  International Conference on Information and CommunicationsSecurity  Springer  http://dblp.uni -trier.de/db/conf/icics/",
  "18 SecureComm  International Conference on Security and Privacy inCommunication Networks  ACM  http://dblp.uni -trier.de/db/conf/securecomm/",
  "19 NSPW  New Security Paradigms Workshop  ACM  http://dblp.uni -trier.de/db/conf/nspw/",
  "20 PAM  Passive and Active Measurement Conference  Springer  http://dblp.uni -trier.de/db/conf/pam/",
  "21 PETS  Privacy Enhancing Technologies Symposium  Springer  http://dblp.uni -trier.de/db/conf/pet/",
  "22 SAC  Selected Areas in Cryptography  Springer  http://dblp.uni -trier.de/db/conf/sacrypt/",
  "23 SOUPS  Symposium On Usable Privacy and Security  USENIX  http://dblp.uni -trier.de/db/conf/soups/",
  "24 HotSec  USENIX Workshop on Hot Topics in Security  USENIX  *http://www.usenix.org/events/",
  "1 PEPM  ACM SIGPLAN Workshop on Partial Evaluation and ProgramManipulation  ACM  http://dblp.uni -trier.de/db/conf/pepm/",
  "2 PASTE  ACMSIGPLAN -SIGSOFT Workshop on Program Analysisfor Software Tools and Engineering  ACM  http://dblp.uni -trier.de/db/conf/paste/",
  "3 APLAS  Asian Symposium on Programming Languages and Systems  Springer  http://dblp.uni -trier.de/db/conf/aplas/",
  "4 APSEC  Asia -Pacific Software Engineering Conference  IEEE  http://dblp.uni -trier.de/db/conf/apsec/",
  "5 EASE  Evaluation and Assessment in Software Engineering  ACM  http://dblp.uni -trier.de/db/conf/ease/",
  "6 ICECCS  IEEE International Conference on Engineering of ComplexComputer Systems  IEEE  http://dblp.uni -trier.de/db/conf/iceccs/",
  "7 ICST  IEEE International Conference on Software Testing,Verification and Validation  IEEE  http://dblp.uni -trier.de/db/conf/icst/",
  "8 ISPASS  IEEE International Symposium on Performance Analysis ofSystems and Software  IEEE  http://dblp.uni -trier.de/db/conf/ispass/",
  "9 SCAM  IEEE International Working Conference on Source CodeAnalysis and Manipulation  IEEE  http://dblp.uni -trier.de/db/conf/scam/",
  "10 COMPSAC  International Computer Software and Applications Conference  IEEE  http://dblp.uni -trier.de/db/conf/compsac/  ",
  "11 ICFEM  International Conference on Formal Engineering Methods  Springer  http://dblp.uni -trier.de/db/conf/icfem/",
  "12 TOOLS  International Conference on Objects, Models, Components,Patterns  Springer  http://dblp.uni -trier.de/db/conf/tools/index.html",
  "13 SCC  International Conference on Service Computing  IEEE  http://dblp.uni -trier.de/db/conf/IEEEscc/",
  "14 ICSSP  International Conference on Software and System Process  ACM  http://dblp.uni -trier.de/db/conf/ispw/",
  "15 SEKE  International Conference on Software Engineering andKnowledge Engineering  KSI  http://dblp.uni -trier.de/db/conf/seke/",
  "16 QRS  International Conference on Software Quality, Reliability andSecurity  IEEE  https://dblp.uni -trier.de/db/conf/qrs/",
  "17 ICSR  International Conference on Software Reuse  Springer  http://dblp.uni -trier.de/db/conf/icsr/",
  "18 ICWE  International Conference on Web Engineering  Springer  http://dblp.uni -trier.de/db/conf/icwe/",
  "19 SPIN  International SPIN Workshop on Model Checking Software  Springer  http://dblp.uni -trier.de/db/conf/spin/index.html",
  "20 ATVA  International Symposium on Automated Technology forVerification and Analysis  Springer  http://dblp.uni -trier.de/db/conf/atva/",
  "21 LOPSTR  International Symposium on Logic -based Program Synthesisand Transformation  Springer  http://dblp.uni -trier.de/db/conf/lopstr/",
  "22 TASE  International Symposium on Theoretical Aspects of SoftwareEngineering  IEEE  http://dblp.uni -trier.de/db/conf/tase/",
  "23 MSR  Mining Software Repositories  IEEE/ACM  http://dblp.uni -trier.de/db/conf/msr/",
  "24 REFSQ  Requirements Engineering: Foundation for Software Quality  Springer  http://dblp.uni -trier.de/db/conf/refsq/",
  "25 WICSA  Working IEEE/IFIP Conference on Software Architecture  IEEE  http://dblp.uni -trier.de/db/conf/wicsa/  ",
  "1 APWeb  Asia Pacific Web Conference  Springer  http://dblp.uni -trier.de/db/conf/apweb/",
  "2 DEXA  Database and Expert System Applications  Springer  http://dblp.uni -trier.de/db/conf/dexa/",
  "3 ECIR  European Conference on IR Research  Springer  http://dblp.uni -trier.de/db/conf/ecir/",
  "4 ESWC  Extended Semantic Web Conference  Springer  http://dblp.uni -trier.de/db/conf/esws/",
  "5 WebDB  International ACM Workshop on Web andDatabases  ACM  http://dblp.uni -trier.de/db/conf/webdb/",
  "6 ER  International Conference on ConceptualModeling  Springer  http://dblp.uni -trier.de/db/conf/er/",
  "7 MDM  International Conference on Mobile DataManagement  IEEE  http://dblp.uni -trier.de/db/conf/mdm/",
  "8 SSDBM  International Conference on Scientific andStatistical DB Management  IEEE  http://dblp.uni -trier.de/db/conf/ssdbm/",
  "9 WAIM  International Conference on Web AgeInformation Management  Springer  http://dblp.uni -trier.de/db/conf/waim/",
  "10 SSTD  International Symposium on Spatial andTemporal Databases  Springer  http://dblp.uni -trier.de/db/conf/ssd/",
  "11 PAKDD  Pacific -Asia Conference on KnowledgeDiscovery and Data Mining  Springer  http://dblp.uni -trier.de/db/conf/pakdd/",
  "12 WISE  Web Information Systems Engineering  Springer  http://dblp.uni -trier.de/db/conf/wise/",
  "1 CSL  Computer Science Logic  Springer  http://dblp.uni -trier.de/db/conf/csl/",
  "2 FMCAD  Formal Methods in Computer -Aided Design  ACM  http://dblp.uni -trier.de/db/conf/fmcad/",
  "3 FSTTCS  Foundations of Software Technology andTheoretical Computer Science  Indian Association for Researchin Computing Science  http://dblp.uni -trier.de/db/conf/fsttcs/",
  "4 DSAA  IEEE International Conference on Data Scienceand Advanced Analytics  IEEE  https://dblp.uni -trier.de/db/conf/dsaa/",
  "5 ICTAC  International Colloquium on Theoretical Aspectsof Computing  Springer  http://dblp.uni -trier.de/db/conf/ictac/",
  "6 IPCO  International Conference on Integer Programmingand Combinatorial Optimization  Springer  http://dblp.uni -trier.de/db/conf/ipco/",
  "7 RTA  International Conference on RewritingTechniques and Applications  Springer  http://dblp.uni -trier.de/db/conf/rta/",
  "8 ISAAC  International Symposium on Algorithms andComputation  Springer  http://dblp.uni -trier.de/db/conf/isaac/",
  "9 MFCS  Mathematical Foundations of Computer Science  Springer  http://dblp.uni -trier.de/db/conf/mfcs/",
  "10 STACS  Symposium on Theoretical Aspects of ComputerScience  Springer  http://dblp.uni -trier.de/db/conf/stacs/",
  "1   ACM Symposium on Virtual RealitySoftware and Technology  ACM  http://dblp2.uni -trier.de/db/conf/vrst/",
  "2 CASA  Computer Animation and Social Agents  Wiley  http://dblp.uni -trier.de/db/conf/ca/",
  "3 CGI  Computer Graphics International  Springer  http://dblp.uni -trier.de/db/conf/cgi/",
  "4 INTERSPEECH  Conference of the International SpeechCommunication Association    http://dblp.uni -trier.de/db/conf/interspeech/index.html",
  "5 GMP  Geometric Modeling and Processing  Elsevier  http://dblp.uni -trier.de/db/conf/gmp/",
  "6 PacificVis  IEEE Pacific Visualization Symposium  IEEE  http://dblp.uni -trier.de/db/conf/apvis/",
  "7 3DV  International Conference on 3D Vision  IEEE  https://dblp.uni -trier.de/db/conf/3dim/",
  "8 CAD/Graphics  International Conference on Computer -AidedDesign and Computer Graphics  IEEE  https://dblp.uni -trier.de/db/conf/cadgraphics/",
  "9 ICIP  International Conference on Image Processing  IEEE  http://dblp.uni -trier.de/db/conf/icip/",
  "10 MMM  International Conference on MultimediaModeling  Springer  http://dblp.uni -trier.de/db/conf/mmm/index.html",
  "11 PCM  Pacific -Rim Conference on Multimedia  Springer  http://dblp.uni -trier.de/db/conf/pcm/",
  "12 SMI  Shape Modeling International  IEEE  http://dblp.uni -trier.de/db/conf/smi/  ",
  "1 AISTATS  Artificial Intelligence and Statistics  JMLR  http://dblp.uni -trier.de/db/conf/aistats/",
  "2 ACCV  Asian Conference on Computer Vision  Springer  http://dblp.uni -trier.de/db/conf/accv/",
  "3 ACML  Asian Conference on MachineLearning  JMLR  http://dblp.uni -trier.de/db/conf/acml/",
  "4 BMVC  British Machine Vision Conference   British Machine VisionAssociation  http://dblp.uni -trier.de/db/conf/bmvc/",
  "5 NLPCC  CCF International Conference onNatural Language Pro cessing andChinese Computing  Springer  https://dblp.uni -trier.de/db/conf/nlpcc/",
  "6 CoNLL  Conference on Computational NaturalLanguage Learning  Association forComputational Linguistics  http://dblp.uni -trier.de/db/conf/conll",
  "7 GECCO  Genetic and Evolutionary ComputationConference  ACM  http://dblp.uni -trier.de/db/conf/gecco/",
  "8 ICTAI  IEEE International Conference onTools with Artificial Intelligence  IEEE  http://dblp.uni -trier.de/db/conf/ictai/",
  "9 IROS  IEEE \\RSJ International Conference onIntelligent Robots and Systems  IEEE  http://dblp.uni -trier.de/db/conf/iros/",
  "10 ALT  International Conference onAlgorithmic Learning Theory  Springer  http://dblp.uni -trier.de/db/conf/alt/",
  "11 ICANN  International Conference on ArtificialNeural Networks  Springer  http://dblp.uni -trier.de/db/conf/icann/",
  "12 FG  International Conference on AutomaticFace and Gesture Recognition  IEEE  http://dblp.uni -trier.de/db/conf/fgr/  ",
  "13 ICDAR  International Conference on DocumentAnalysis and Recognition  IEEE  http://dblp.uni -trier.de/db/conf/icdar/",
  "14 ILP  International Conference on InductiveLogic Programming  Springer  http://dblp.uni -trier.de/db/conf/ilp/",
  "15 KSEM  International conference on KnowledgeScience,  Engineering and Management  Springer  http://dblp.uni -trier.de/db/conf/ksem/",
  "16 ICONIP  International Conference on NeuralInformation Processing  Springer  http://dblp.uni -trier.de/db/conf/iconip/",
  "17 ICPR  International Conference on PatternRecognition  IEEE  http://dblp.uni -trier.de/db/conf/icpr/",
  "18 ICB  International Joint Conference onBiometrics  IEEE  http://dblp.uni -trier.de/db/conf/icb/",
  "19 IJCNN  International Joint Conference onNeural Networks  IEEE  http://dblp.uni -trier.de/db/conf/ijcnn/",
  "20 PRICAI  Pacific Rim International Conferenceon Artificial Intelligence  Springer  http://dblp.uni -trier.de/db/conf/pricai/",
  "21 NAACL  The Annual Conference of the NorthAmerican Chapter of the Association for Computational Linguistics  NAACL  http://dblp.uni -trier.de/db/conf/naacl/  ",
  "1 DIS  ACM Conference on Designing Interactive Systems  ACM  http://dblp.uni -trier.de/db/conf/ACMdis",
  "2 ICMI   ACM International Conference on Multimodal Interaction  ACM  http://dblp.uni -trier.de/db/conf/icmi/",
  "3 ASSETS  ACM SIGACCESS Conference on Computers and Accessibility  ACM  http://dblp.uni -trier.de/db/conf/assets",
  "4 GI Graphics Interface conference  ACM  http://dblp.uni -trier.de/db/conf/graphicsinterface/",
  "5 UIC  IEEE International Conference on Ubiquitous Intelligence andComputing  IEEE  http://dblp.uni -trier.de/db/conf/uic/",
  "6  IEEE World Haptics Conference  IEEE  http://dblp.uni -trier.de/db/conf/haptics/",
  "7 INTERACT  IFIP TC13 Conference on Human -Computer Interaction  IFIP  http://dblp.uni -trier.de/db/conf/interact/",
  "8 IDC  Interaction Design and Children  ACM  http://dblp.uni -trier.de/db/conf/acmidc/",
  "9 CollaborateCom  International Conference on Collaborative Computing:Networking, Applications and Worksharing  Springer  http://collaboratecom.org",
  "10 CSCWD  International Conference on Computer Supported CooperativeWork in Design  Springer  http://dblp.uni -trier.de/db/conf/cscwd/",
  "11 CoopIS  International Conference on Cooperative Information Systems  Springer  http://dblp.uni -trier.de/db/conf/coopis/",
  "12 MobiQuitous  International Conference on Mobile and Ubiquitous Systems:Computing, Networking and Services  Springer  http://dblp.uni -trier.de/db/conf/mobiquitous/",
  "13 AVI  International Working Conference on Advanced Visual Interfaces  ACM  http://dblp.uni -trier.de/db/conf/avi/  ",
  "1 AMIA  American Medical Informatics AssociationAnnual Symposium  AMIA  http://dblp.uni -trier.de/db/conf/amia/",
  "2 APBC  Asia Pacific Bioinformatics Conference  BioMed Central  http://dblp.uni -trier.de/db/conf/apbc/",
  "3  IEEE International Conference on Big Data  IEEE  https://dblp.uni -trier.de/db/conf/bigdataconf/",
  "4  IEEE International Conference on CloudComputing  IEEE  http://dblp.uni -trier.de/db/conf/IEEEcloud/",
  "5 SMC  IEEE International Conference on Systems,Man, and Cybernetics  IEEE  https://dblp.uni -trier.de/db/conf/smc/",
  "6 COSIT  International Conference on Spatial InformationTheory  ACM  http://dblp.uni -trier.de/db/conf/cosit/",
  "7 ISBRA  International Symposium on BioinformaticsResearch and Applications  Springer  https://dblp.uni -trier.de/db/conf/isbra/"
]


import asyncio
from .data import CCF_A, CCF_B, CCF_C
from .downloader import *
from typing import Optional, Iterator
import re
import logging

logger = logging.getLogger("parser")


class Person:
    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "person", "Should be xml of a person in dblpperson!"
        self.data = data

    def name(self) -> Optional[str]:
        for child in self.data:
            if child.tag == "author":
                return child.text
        return None

    def affiliations(self) -> Iterator[str]:
        for child in self.data:
            if child.tag == "note" and "type" in child.attrib and child.attrib["type"] == "affiliation":
                if child.text is not None:
                    yield child.text


class Author:
    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "author", "Should be xml of a author in someone's publication!"
        self.data = data

    def name(self) -> str:
        return self.data.text if self.data.text is not None else ''

    def pid(self) -> str:
        return self.data.attrib['pid']

    async def dblpperson(self):
        data = await download_person(self.pid())
        if data is None:
            return None
        return DBLPPerson(data)


class Publication:
    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "r", "Should be xml of a <r> tag in dblpperson!"
        self.data = data[0]

    def key(self) -> str:
        return self.data.attrib["key"]

    def authors(self) -> Iterator[Author]:
        for child in self.data:
            if child.tag == "author":
                yield Author(child)

    def title(self) -> Optional[str]:
        for child in self.data:
            if child.tag == "title":
                return " ".join(t for t in child.itertext())
        return None

    def journal(self) -> Optional[str]:
        tag = {
            'inproceedings': 'booktitle',
            'proceedings': 'booktitle',
            'article': 'journal',
            'incollection': 'booktitle',
            'book': 'series'
        }
        for child in self.data:
            if self.data.tag in tag and child.tag == tag[self.data.tag]:
                return child.text
        return None

    def journal_key(self) -> Optional[str]:
        for child in self.data:
            if child.tag == "url":
                return "/".join(child.text.split("/")[0:3]) if child.text is not None else None
        return None

    def year(self) -> Optional[int]:
        for child in self.data:
            if child.tag == "year":
                return int(child.text) if child.text is not None else None
        return None

    def ee(self) -> Iterator[str]:
        for child in self.data:
            if child.tag == "ee":
                if child.text is not None:
                    yield child.text

    def doi(self) -> Optional[str]:
        for e in self.ee():
            if re.search(r'doi\.org', e) is not None:
                return e
        return None

    def ccf(self) -> str:
        j = self.journal_key()
        for (ccf, ls) in zip(['A', 'B', 'C'], [CCF_A, CCF_B, CCF_C]):
            if j in ls:
                return ccf
        return 'N'

    def __dict__(self) -> dict:
        return dict(
            key=self.key(),
            title=self.title(),
            journal=self.journal(),
            journal_key=self.journal_key(),
            year=self.year(),
            doi=self.doi(),
            ccf=self.ccf(),
            authors=", ".join([author.name() for author in self.authors()])[0:-2],
            authors_pid=[author.pid() for author in self.authors()],
        )


class DBLPPerson:
    ID = 0

    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "dblpperson", "Should be xml of a dblpperson!"
        self.data = data
        logger.debug(f"<{self.data.tag} %s>" % " ".join("%s=\"%s\"" % (k, v) for k, v in self.data.attrib.items()))
        DBLPPerson.ID += 1
        self.id = DBLPPerson.ID

    def pid(self) -> str:
        return self.data.attrib['pid'] if 'pid' in self.data.attrib else ("Unknown_%d" % self.id)

    def name(self) -> str:
        return self.data.attrib['name'] if 'name' in self.data.attrib else ("Unname_%d" % self.id)

    def person(self) -> Optional[Person]:
        for child in self.data:
            if child.tag == "person":
                return Person(child)
        return None

    def publications(self) -> Iterator[Publication]:
        for child in self.data:
            if child.tag == "r":
                yield Publication(child)

    def __dict__(self) -> dict:
        person = self.person()
        return dict(
            pid=self.pid(),
            name=self.name(),
            affiliations=list(person.affiliations()),
            publications=[pub.key() for pub in self.publications()]
        )


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)


    async def main() -> None:
        print(DBLPPerson(await download_person('74/1552-1')))


    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



def get_publication(summary, key):
    if key in summary["publications"]:
        return summary["publications"][key]
    return None


def foreach_publication(summary, key):
    if key in summary["publications"]:
        return summary["publications"][key]
    return None


def map_dict(d, callback):
    md = {}
    for k, v in d.items():
        mv = callback(k, v)
        if mv is not None:
            md[k] = mv
    return md


def map_publications(summary, callback):
    summary["publications"] = map_dict(summary["publications"], callback)
    return summary


def map_node(summary, callback):
    summary["nodes"] = map_dict(summary["nodes"], lambda k, v: callback(k, v))
    edges = {}
    for k, edge in summary["edges"].items():
        if edge['from'] in summary["nodes"] and edge['to'] in summary["nodes"]:
            edges[k] = edge
    summary["edges"] = edges
    return summary


def map_person(summary, callback):
    def callback_node(k, node):
        node["person"] = callback(k, node["person"])
        return node

    return map_node(summary, callback_node)


def map_person_publications(summary, callback):
    def callback_person(_, person):
        publications = person["publications"]
        mapped_publications = []
        for key in publications:
            if key not in summary["publications"]:
                continue
            publication = callback(person, summary["publications"][key])
            if publication is None:
                continue
            mapped_publications.append(publication["key"])
        person["publications"] = mapped_publications
        return person

    return map_person(summary, callback_person)


def map_edge(summary, callback):
    summary["edges"] = map_dict(summary["edges"], callback)
    return summary


def map_cooperation(summary, callback):
    def callback_edge(_, edge):
        publications = edge["cooperation"]
        mapped_publications = []
        for key in publications:
            if key not in summary["publications"]:
                continue
            publication = callback(edge, summary["publications"][key])
            if publication is None:
                continue
            mapped_publications.append(publication["key"])
        edge["cooperation"] = mapped_publications
        return edge

    return map_edge(summary, callback_edge)



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

with open("README.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

package_dir = {
    'dblp_crawler': 'dblp_crawler',
    'dblp_crawler.filter': 'dblp_crawler/filter',
    'dblp_crawler.data': 'dblp_crawler/data',
    'dblp_crawler.data.ccf': 'dblp_crawler/data/ccf',
    'dblp_crawler.keyword': 'dblp_crawler/keyword',
    'dblp_crawler.summarizer': 'dblp_crawler/summarizer',
    'dblp_crawler.summarizer.networkx': 'dblp_crawler/summarizer/networkx',
    'dblp_crawler.summarizer.neo4j': 'dblp_crawler/summarizer/neo4j',
}

setup(
    name='dblp_crawler',
    version='1.6.2.1',
    author='yindaheng98',
    author_email='yindaheng98@163.com',
    url='https://github.com/yindaheng98/dblp-crawler',
    description=u'异步高并发dblp爬虫，慎用',
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir=package_dir,
    packages=[key for key in package_dir],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'networkx==2.*',
        'aiofile>=3.8.1',
        'aiohttp>=3.8.1',
        'neo4j>=5.5.0',
    ],
)



import logging
from typing import AsyncIterator
import xml.etree.ElementTree as ElementTree
from .parser import Publication
from .downloader import download_journal

logger = logging.getLogger("parser")


class Journal:
    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "bht", "Should be xml of a bht!"
        self.data = data

    def publications(self) -> list[Publication]:
        return [Publication(r) for r in self.data.findall('./dblpcites/r')]


class JournalList:
    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "bht", "Should be xml of a bht!"
        self.data = data

    def title(self) -> str:
        return self.data.attrib['title']

    def journal_keys(self) -> list[str]:
        v1 = [li.attrib["href"] for li in self.data.findall('./ul/li/ref')]
        h1 = self.data.find('./h1').text
        v2 = [proceedings.find('./url').text
              for proceedings in self.data.findall('./dblpcites/r/proceedings')
              if proceedings.find('./booktitle') is None or proceedings.find('./booktitle').text == h1]
        return v1 + v2

    async def journals(self) -> AsyncIterator[Journal]:
        for jid in self.journal_keys():
            data = await download_journal(jid)
            if data is not None:
                yield Journal(data)

    async def publications(self) -> AsyncIterator[Publication]:
        async for j in self.journals():
            for p in j.publications():
                yield p


if __name__ == "__main__":
    import asyncio
    from dblp_crawler import download_journal_list


    async def main() -> None:
        data = await download_journal_list('db/journals/tmm')
        if data is None:
            return
        jl = JournalList(data)
        i = 0
        async for publication in jl.publications():
            i += 1
            if i >= 100:
                break
            print(publication)


    asyncio.run(main())
